# coding=utf8

import re

import unidecode

from . import classes

#############################################################################################################
##############################           funcao        get_paths         ####################################
#############################################################################################################


def get_paths():
    p = classes.Paths()
    with open("../paths.cnf", "r") as f:
        for linha in f:
            tok = linha.split()
            if "GRUPOSPATH" == tok[0]:
                p.GRUPOSPATH = tok[1]
            if "PREFPATH" == tok[0]:
                p.PREFPATH = tok[1]
            if "SARPATH" == tok[0]:
                p.SARPATH = tok[1]
            if "ATRIBPATH" == tok[0]:
                p.ATRIBPATH = tok[1]
            if "FANPATH" == tok[0]:
                p.FANTPATH = tok[1]
            if "DATPATH" == tok[0]:
                p.DATPATH = tok[1]
            if "SOLPATH" == tok[0]:
                p.SOLPATH = tok[1]
    if (
        (p.GRUPOSPATH != None)
        and (p.PREFPATH != None)
        and (p.SARPATH != None)
        and (p.ATRIBPATH != None)
        and (p.FANTPATH != None)
        and (p.DATPATH != None)
        and (p.SOLPATH != None)
    ):
        p.preenchida = True
    return p


#############################################################################################################
##############################           funcao        compara_nomes     ####################################
#############################################################################################################
def compara_nomes(
    nome1, nome2
):  # Diz se os nomes se referem a mesma pessoa, uniformizacao e necessaria eh retornada resposta positiva se as pessoas possuem dois nomes em comum
    tok1 = nome1.split()
    tok2 = nome2.split()
    ocorrencia = False
    for i in range(0, len(tok1)):
        for j in range(0, len(tok2)):
            if len(tok1[i]) > 0 and tok1[i] == tok2[j]:
                ococrrencia = True
                break
    if not ococrrencia:
        return False
    for k in range(i + 1, len(tok1)):
        for l in range(0, len(tok2)):
            if len(tok1[i]) > 0 and tok1[k] == tok2[l]:
                return True
    return False


##
# Funcao que uniformiza strings: retira acentos e deixa todas as letras maiusculas.
##
def uniformize(s, encode="utf8"):
    return re.sub("[.']", "", unidecode.unidecode(s).upper().strip())



import datetime
import os

import pandas as pd

from . import funcoes_leitura as leitura


def gera_declaracao_ch_semestre(
    listah, s, dirh="/home/fsobral/GoogleDrive/Alforria/2024/horarios"
):
    """Assume que `s` é 'S1' ou 'S2' e `listah` é uma lista de inteiros que representam datas na forma YYYYMMDD, e que contem o primeiro dia de aula do semestre e o ultimo. Faz mais sentido arrendondar por semanas e sempre iniciar na segunda e terminar na segunda, para facilitar a vida. Lembrando que o número de semanas tem que dar 17

    Exemplo:

        listah([20240514, 20240527, 20240610, 20240708, 20240722, 20240805, 20240826, 20240909], 'S1')

    """

    # Calcula a quantidade de semanas entre cada data
    datas = sorted(listah)

    ch = [
        (
            (
                datetime.date.fromisoformat(str(datas[i + 1]))
                - datetime.date.fromisoformat(str(datas[i]))
            ).days
            + 1
        )
        // 7
        for i in range(0, len(datas) - 1)
    ]

    print("Numero de semanas total:", sum(ch))

    # Nao deveria existir pasta com a ultima semana
    v = datas[0:-1]

    # Junta todas as disciplinas que foram ministradas no intervalo dado
    dfdisc = (
        pd.concat(
            [
                pd.read_csv(
                    "{1:s}/{0:d}/atribuicoes.csv".format(d, dirh), sep=",", header=None
                )
                for d in v
            ]
        )
        .groupby([1, 2, 3, 6])
        .agg({4: "first"})
    )

    # Calcula a carga horária total

    soma_ch = pd.concat(
        [
            pd.read_csv(
                "{1:s}/{0:d}/atribuicoes.csv".format(d, dirh), sep=",", header=None
            ).set_index([1, 2, 3, 6])[5]
            * c
            for d, c in zip(v, ch)
        ],
        axis=1,
    )

    dfdisc.loc[:, "CH_MINISTRADA"] = soma_ch.sum(axis=1)

    return dfdisc.loc[:, :, :, s]


def sar_to_csv(turmas, outfile):
    """
    Converte a planilha SAR para um CSV.
    """

    with open(outfile, "w") as fp:
        for t in turmas:
            for h in t.horarios:
                fp.write(
                    "{cod:s},{turma:s},{nome:s},T,99,9,{horario[0]:d},{horario[1]:d},9,CAMPUS,BLOCO,9,{ch:.1f},{cht:.1f},S{semestre:d},{ini:%Y-%m-%d},{fim:%Y-%m-%d},{curso:s}\n".format(
                        cod=t.codigo,
                        turma=t.turma,
                        nome=t.nome,
                        horario=h,
                        ch=t.ch,
                        cht=(t.ch * 17),
                        semestre=t.semestralidade,
                        ini=t.dini,
                        fim=t.dend,
                        curso=t.curso,
                    )
                )


def write_ch_table(professor, diretorio, csv_name="ch_table.csv"):
    """
    Create a .csv file with the teaching hours in each semester for each professor.
    """

    if not os.path.exists(diretorio):
        os.makedirs(diretorio)

    with open(diretorio + csv_name, "w") as fp:
        for p in professor:
            fp.write(
                "{0:s},{1:d},{2:d}\n".format(
                    p.nome(), p._carga_horaria_s(1), p._carga_horaria_s(2)
                )
            )


def cria_relatorio(professor, diretorio):
    """
    Recebe um professor e cria um arquivo .tex com o seu relatorio.
    """

    if not os.path.exists(diretorio):
        os.makedirs(diretorio)

    with open("../config/base.tex", "r") as padrao:
        with open(diretorio + "relatorio_" + professor.nome() + ".tex", "w") as saida:
            professor.att_totex(diretorio + professor.nome() + ".tex")
            for l in padrao:
                saida.write(l)
            saida.write("\\include{" + professor.nome() + "}")
            saida.write("\\end{document}")


def cria_relatorio_geral(professores, diretorio):
    """
    Recebe uma lista de objetos do tipo Professor e constroi um arquivo .tex
    com todos os relatorios. Utiliza um arquivo chamado 'base.tex' como base
    para carregar os pacotes e similares.
    """

    if not os.path.exists(diretorio):
        os.makedirs(diretorio)

    count = 1
    with open("../config/base.tex", "r") as padrao:
        with open(diretorio + "relatorio_geral.tex", "w") as saida:
            for l in padrao:
                saida.write(l)
            for p in professores:
                p.totex(diretorio + str(count) + ".tex")
                saida.write("\\include{" + str(count) + "}\n")
                count += 1
            saida.write("\\end{document}")


####################################################################################################################
####################                  Funcao              escreve dat                      #########################
####################################################################################################################


def escreve_dat(professores, turmas, grupos, pre_atribuidas, arquivo):
    with open(arquivo, "w") as f:
        constantes = leitura.ler_conf("../config/constantes.cnf")

        for param in constantes:
            f.write("param " + param + " := " + constantes[param] + ";\n\n")

        f.write("set G := ")
        for g in grupos:
            f.write(g.id + " ")
        f.write(";\n\n")

        f.write("set G_CANONICOS := ")
        for g in grupos:
            if g.canonico:
                f.write(g.id + " ")
        f.write(";\n\n")

        f.write("set T := ")
        for t in turmas:
            f.write(t.id() + " ")
        f.write(";\n\n")

        f.write("set T_PRE := ")
        for p, t in pre_atribuidas:
            f.write(t.id() + " ")
        f.write(";\n\n")

        f.write("set P := ")
        for p in professores:
            f.write(p.id() + " ")
        f.write(";\n\n")

        # Vincula as turmas ao grupos
        f.write("param turma_grupo := \n")
        for t in turmas:
            if t.grupo is not None:
                f.write(t.id() + " " + t.grupo.id + " 1\n")
        f.write(";\n\n")

        # Supoe que se uma disciplina eh vinculada, seu vinculo eh a
        # disciplina seguinte
        f.write("param vinculadas := \n")
        vinc = False
        for t in turmas:
            if vinc:
                f.write(t.id() + " 1\n")
                vinc = False
            elif t.vinculada:
                f.write(t.id() + " ")
                vinc = True
        f.write(";\n\n")

        f.write("param c := \n")
        for t in turmas:
            for d, h in t.horarios:
                f.write(
                    t.id()
                    + " "
                    + str(t.semestralidade)
                    + " "
                    + str(d)
                    + " "
                    + str(h)
                    + " 1\n"
                )
        f.write(";\n\n")

        f.write("param ch := \n")
        for t in turmas:
            f.write(t.id() + " " + str(t.carga_horaria()) + "\n")
        f.write(";\n\n")

        f.write("param ch1 := \n")
        for t in turmas:
            f.write(t.id() + " ")
            if t.semestralidade == 1:
                f.write(str(t.carga_horaria()) + "\n")
            else:
                f.write("0\n")
        f.write(";\n\n")

        f.write("param ch2 := \n")
        for t in turmas:
            f.write(t.id() + " ")
            if t.semestralidade == 2:
                f.write(str(t.carga_horaria()) + "\n")
            else:
                f.write("0\n")
        f.write(";\n\n")

        f.write("param temporario := \n")
        for p in professores:
            if p.temporario:
                f.write(p.id() + " 1\n")
        f.write(";\n\n")

        f.write("param inapto := \n")
        for p in professores:
            for g in p.inapto:
                f.write(p.id() + " " + g + " 1\n")
        f.write(";\n\n")

        f.write("param impedimento := \n")
        for p in professores:
            for d in range(2, 8):
                for h in range(1, 17):
                    if p.impedimentos[h][d]:
                        for s in range(1, 3):
                            f.write(
                                p.id()
                                + " "
                                + str(s)
                                + " "
                                + str(d)
                                + " "
                                + str(h)
                                + " 1\n"
                            )
        f.write(";\n\n")

        f.write("param pref_janelas := \n")
        for p in professores:
            if p.pref_janelas:
                f.write(p.id() + " 1\n")
        f.write(";\n\n")

        f.write("param pref_grupo := \n")
        for p in professores:
            for g in p.pref_grupos.keys():
                f.write(p.id() + " " + g + " " + str(p.pref_grupos[g]) + "\n")
        f.write(";\n\n")

        f.write("param pref_hor := \n")
        for p in professores:
            for d in range(2, 8):
                for h in range(1, 17):
                    f.write(
                        p.id()
                        + " "
                        + str(d)
                        + " "
                        + str(h)
                        + " "
                        + str(p.pref_horarios[h, d])
                        + "\n"
                    )
        f.write(";\n\n")

        f.write("param pre_atribuida :=\n")
        for p, t in pre_atribuidas:
            f.write(p.id() + " " + t.id() + " 1\n")
        f.write(";\n\n")

        f.write("param chprevia1 :=\n")
        for p in professores:
            if p.chprevia1 > 0:
                f.write(p.id() + " " + str(p.chprevia1) + "\n")
        f.write(";\n\n")

        f.write("param chprevia2 :=\n")
        for p in professores:
            if p.chprevia2 > 0:
                f.write(p.id() + " " + str(p.chprevia2) + "\n")
        f.write(";\n\n")

        f.write("param licenca :=\n")
        for p in professores:
            if p.licenca1:
                f.write(p.id() + " 1 1\n")
            if p.licenca2:
                f.write(p.id() + " 2 1\n")
        f.write(";\n\n")

        f.write("param peso_disciplinas := \n")
        for p in professores:
            f.write(p.id() + " " + str(p.peso_disciplinas) + "\n")
        f.write(";\n\n")

        f.write("param peso_horario := \n")
        for p in professores:
            f.write(p.id() + " " + str(p.peso_horario) + "\n")
        f.write(";\n\n")

        f.write("param peso_cargahor := \n")
        for p in professores:
            f.write(p.id() + " " + str(p.peso_cargahor) + "\n")
        f.write(";\n\n")

        f.write("param peso_distintas := \n")
        for p in professores:
            f.write(p.id() + " " + str(p.peso_distintas) + "\n")
        f.write(";\n\n")

        f.write("param peso_janelas := \n")
        for p in professores:
            f.write(p.id() + " " + str(p.peso_janelas) + "\n")
        f.write(";\n\n")

        f.write("param peso_numdisc := \n")
        for p in professores:
            f.write(p.id() + " " + str(p.peso_numdisc) + "\n")
        f.write(";\n\n")

        f.write("param peso_manha_noite := \n")
        for p in professores:
            f.write(p.id() + " " + str(p.peso_manha_noite) + "\n")
        f.write(";\n\n")

        f.write("param chmax := \n")
        for p in professores:
            if p.chmax != None:
                f.write(p.id() + " " + str(p.chmax) + "\n")
        f.write(";\n\n")

        f.write("param chmax1 := \n")
        for p in professores:
            if p.chmax1 != None:
                f.write(p.id() + " " + str(p.chmax1) + "\n")
        f.write(";\n\n")

        f.write("param chmax2 := \n")
        for p in professores:
            if p.chmax2 != None:
                f.write(p.id() + " " + str(p.chmax2) + "\n")
        f.write(";\n\nend;\n")


####################################################################################################################
####################                  Funcao              escreve atribuicoes              #########################
####################################################################################################################


def escreve_pre_atribuidas(professores, turmas, arquivo):
    """
    Esta funcao simplesmente salva a atribuicao atual.
    """

    with open(arquivo, "w") as f:
        for p in professores:
            for t in p.turmas_a_lecionar:
                # TODO: trocar p.matricula por p.id()
                sem = "S" + str(t.semestralidade)

                f.write(
                    "{matricula:08d}\t{nome:s}\t{cod:s}\t{turma:s}\t{tnome:s}\t{ch:d}\t{sem:s}\t{curso:s}\n".format(
                        matricula=p.matricula,
                        nome=p.nome(),
                        cod=t.codigo,
                        turma=t.turma,
                        tnome=t.nome,
                        ch=t.carga_horaria(),
                        sem=sem,
                        curso=t.curso,
                    )
                )


def escreve_atribuicoes(professores, turmas, arquivo):
    """This function writes in a CSV file 'arquivo' the relation of
    courses attributed to each professor.

    """

    with open(arquivo, "w") as f:
        for p in professores:
            for t in p.turmas_a_lecionar:
                # TODO: Codigo virar numero e nao texto
                if t.codigo.find("666") == 0:
                    continue

                # TODO: trocar p.matricula por p.id()
                sem = "S" + str(t.semestralidade)
                # Agora permitimos disciplinas anuais desvinculadas
                # if t.vinculada and t.semestralidade == 1:
                #     sem = 'A'
                # if t.vinculada and t.semestralidade == 2:
                #     continue
                f.write(
                    "{matricula:08d},{nome:s},{cod:s},{turma:s},{tnome:s},{ch:d},{sem:s},{curso:s}\n".format(
                        matricula=p.matricula,
                        nome=p.nome(),
                        cod=t.codigo,
                        turma=t.turma,
                        tnome=t.nome,
                        ch=t.carga_horaria(),
                        sem=sem,
                        curso=t.curso,
                    )
                )
                # f.write(str(p.matricula) + '\t' + p.nome() + '\t' + str(t.codigo) + '\t' + \
                #         str(t.turma) + '\t' + t.nome + '\t' + str(t.carga_horaria()) + \
                #         '\t' + sem + '\n')
                # TODO: trocar p.matricula por p.id()
                # for fant in t.turmas_clientes:
                #     f.write(str(p.matricula) + '\t' + p.nome() + '\t' + str(fant.codigo) + '\t' + \
                #             str(fant.turma) + '\t' + fant.nome + '\t' + str(fant.carga_horaria()) + \
                #             '\tS' + str(t.semestralidade) + '\n')


####################################################################################################################
####################                  Funcao              escreve disciplinas              #########################
####################################################################################################################


def escreve_disciplinas(professores, turmas, arquivo):
    with open(arquivo, "w") as f:
        for t in sorted(turmas, key=lambda x: int(x.codigo)):
            f.write(
                "{cod:s},{turma:s},{tnome:s},{nome:s},S{sem:s},{anual:s}\n".format(
                    cod=t.codigo,
                    turma=t.turma,
                    tnome=t.nome,
                    nome="" if t.professor is None else t.professor.nome(),
                    sem=str(t.semestralidade),
                    anual=("*" if t.vinculada else ""),
                )
            )
            # f.write(t.nome + '\t' + str(t.codigo) + '\t' + str(t.turma) + '\t')
            # for p in professores:
            #     if t in p.turmas_a_lecionar:
            #         f.write(p.nome())
            # f.write('\t' + str(t.carga_horaria()) + '\t' + 'S' + str(t.semestralidade))
            # f.write('\n')

            # Removidas as fantasmas
            # for fant in t.turmas_clientes:
            #     f.write(fant.nome + '\t' + str(fant.codigo) + '\t' + str(fant.turma) + '\t')
            #     for p in professores:
            #         if t in p.turmas_a_lecionar:
            #             f.write(p.nome())
            #     f.write('\t' + str(t.carga_horaria()) + '\t' + 'S' + str(t.semestralidade))
            #     f.write('\n')


####################################################################################################################
####################                  Funcao              atualiza dat2                    #########################
####################################################################################################################


def atualiza_dat2(professores, arquivo):
    """
    Esta funcao serve para remover professores e preparar para
    uma nova rodada. A ser incluida na nova versao do alforria
    """

    listap = []

    # Leitura dos professores ja excluidos

    with open(arquivo, "r") as f:
        l = f.readline().split()

        if len(l) > 3:
            for nome in l[3:]:
                for p in professores:
                    if p.nome() == nome:
                        listap.append(p)

    professores.sort(key=lambda x: x.insatisfacao, reverse=True)

    # Procura o proximo insatisfeito que nao foi excluido

    i = 0
    while (i < len(professores)) and (professores[i] in listap):
        i = i + 1

    maxinsat = -100
    if i < len(professores):
        maxinsat = professores[i].insatisfacao
        listap.append(professores[i])

    # Gera o novo arquivo

    print("Professores retirados da funcao objetivo:")
    for p in listap:
        print("\t{0:50s} insat: {1:10.7f}".format(p.nome(), p.insatisfacao))

    with open(arquivo, "w") as f:
        f.write("set P_OUT :=")
        for p in listap:
            f.write(" " + p.nome())
        f.write(" ;\n\n")

        f.write("param ub_insat :=\n")
        for p in listap:
            f.write(p.nome() + " " + str(max(maxinsat, p.insatisfacao)) + "\n")
        f.write(";\n\n")

    # Antes de sair, ordena os professores por nome
    professores.sort(key=lambda x: x.nome())


####################################################################################################################
####################                  Funcao              escreve jl                       #########################
####################################################################################################################


def escreve_jl(professores, turmas, grupos, pre_atribuidas, arquivo):
    with open(arquivo, "w") as f:
        constantes = leitura.ler_conf("../config/constantes.cnf")

        for param in constantes:
            f.write(param + " = " + constantes[param] + "\n\n")

        f.write("G :: Set{String} = Set{String}([\n\n")
        for g in grupos:
            f.write(f'\t"{g.id}",\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("G_CANONICOS :: Set{String} = Set{String}([")
        for g in grupos:
            if g.canonico:
                f.write(f'\t"{g.id}",\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("T :: Set{String} = Set{String}([")
        for t in turmas:
            f.write(f'\t"{t.id()}",\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("T_PRE :: Set{String} = Set{String}([")
        for p, t in pre_atribuidas:
            f.write(f'\t"{t.id()}",\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("P :: Set{String} = Set{String}([")
        for p in professores:
            f.write(f'\t"{p.id()}",\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        # Vincula as turmas ao grupos
        f.write("turma_grupo :: Dict{String, String} = Dict(\n")
        for t in turmas:
            if t.grupo is not None:
                f.write(f'\t"{t.id()}" => "{t.grupo.id}",\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        # Supoe que se uma disciplina eh vinculada, seu vinculo eh a
        # disciplina seguinte
        f.write(
            "vinculadas :: Set{Tuple{String, String}} = Set{Tuple{String,String}}([\n"
        )
        vinc = False
        for t in turmas:
            if vinc:
                f.write(f'"{t.id()}"),\n')
                vinc = False
            elif t.vinculada:
                f.write(f'\t("{t.id()}", ')
                vinc = True
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("c :: Set{Tuple{String, Int64, Int64, Int64}} =\n")
        f.write("Set{Tuple{String, Int64, Int64, Int64}}([")
        for t in turmas:
            for d, h in t.horarios:
                f.write(f'\t("{t.id()}", {t.semestralidade}, {d}, {h}),\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("ch :: Dict{String, Int64} = Dict{String, Int64}(\n")
        for t in turmas:
            f.write(f'\t"{t.id()}" => {t.carga_horaria()},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("ch1 :: Dict{String, Int64} = Dict{String, Int64}(\n")
        for t in turmas:
            f.write(f'\t"{t.id()}" => ')
            if t.semestralidade == 1:
                f.write(f"{t.carga_horaria()},\n")
            else:
                f.write("0,\n")
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("ch2 :: Dict{String, Int64} = Dict{String, Int64}(\n")
        for t in turmas:
            f.write(f'\t"{t.id()}" => ')
            if t.semestralidade == 2:
                f.write(f"{t.carga_horaria()},\n")
            else:
                f.write("0,\n")
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("temporario :: Set{String} = Set{String}([\n")
        for p in professores:
            if p.temporario:
                f.write(f'\t"{p.id()}",\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("inapto :: Set{Tuple{String, String}} = Set{Tuple{String, String}}([\n")
        for p in professores:
            for g in p.inapto:
                f.write(f'\t("{p.id()}", "{g}")\n')
        f.seek(f.tell() - 1, 0)
        f.write("\n])\n\n")

        f.write("impedimento :: Set{Tuple{String, Int64, Int64, Int64}} = \n")
        f.write("Set{Tuple{String, Int64, Int64, Int64}}([\n")
        for p in professores:
            for d in range(2, 8):
                for h in range(1, 17):
                    if p.impedimentos[h][d]:
                        for s in range(1, 3):
                            f.write(f'\t("{p.id()}", {s}, {d}, {h})\n')
        f.seek(f.tell() - 1, 0)
        f.write("\n])\n\n")

        f.write("pref_janelas :: Set{String} = Set{String}([\n")
        for p in professores:
            if p.pref_janelas:
                f.write(f'\t"{p.id()}",\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("pref_grupo :: Dict{Tuple{String, String}, Float16} =\n")
        f.write("Dict{Tuple{String, String}, Float16}(\n")
        for p in professores:
            for g in p.pref_grupos.keys():
                f.write(f'\t("{p.id()}", "{g}") => {p.pref_grupos[g]},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("pref_hor :: Dict{Tuple{String, Int64, Int64}, Float16} =\n")
        f.write("Dict{Tuple{String, Int64, Int64}, Float16}(\n")
        for p in professores:
            for d in range(2, 8):
                for h in range(1, 17):
                    f.write(f'\t("{p.id()}", {d}, {h}) => {p.pref_horarios[h, d]},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("pre_atribuida :: Set{Tuple{String, String}} =\n")
        f.write("Set{Tuple{String, String}}([\n")
        for p, t in pre_atribuidas:
            f.write(f'\t("{p.id()}", "{t.id()}"),\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n])\n\n")

        f.write("chprevia1 :: Dict{String, Int64} = Dict{String, Int64}(\n")
        for p in professores:
            if p.chprevia1 > 0:
                f.write(f'\t"{p.id()}" => {p.chprevia1},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("chprevia2 :: Dict{String, Int64} = Dict{String, Int64}(\n")
        for p in professores:
            if p.chprevia2 > 0:
                f.write(f'\t"{p.id()}" => {p.chprevia2},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("licenca :: Set{Tuple{String, Int64}} = Set{Tuple{String, Int64}}([\n")
        for p in professores:
            if p.licenca1:
                f.write(f'\t("{p.id()}", 1)\n')
            if p.licenca2:
                f.write(f'\t("{p.id()}", 2)\n')
        f.seek(f.tell() - 1, 0)
        f.write("\n])\n\n")

        f.write("peso_disciplinas :: Dict{String, Float16} = Dict{String, Float16}(\n")
        for p in professores:
            f.write(f'\t"{p.id()}" => {p.peso_disciplinas},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("peso_horario  :: Dict{String, Float16} = Dict{String, Float16}(\n")
        for p in professores:
            f.write(f'\t"{p.id()}" => {p.peso_horario},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("peso_cargahor :: Dict{String, Float16} = Dict{String, Float16}(\n")
        for p in professores:
            f.write(f'\t"{p.id()}" => {p.peso_cargahor},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("peso_distintas :: Dict{String, Float16} = Dict{String, Float16}(\n")
        for p in professores:
            f.write(f'\t"{p.id()}" => {p.peso_distintas},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("peso_janelas :: Dict{String, Float16} = Dict{String, Float16}(\n")
        for p in professores:
            f.write(f'\t"{p.id()}" => {p.peso_janelas},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("peso_numdisc :: Dict{String, Float16} = Dict{String, Float16}(\n")
        for p in professores:
            f.write(f'\t"{p.id()}" => {p.peso_numdisc},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("peso_manha_noite :: Dict{String, Float16} = Dict{String, Float16}(\n")
        for p in professores:
            f.write(f'\t"{p.id()}" => {p.peso_manha_noite},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("chmax :: Dict{String, Int64} = Dict{String, Int64}(\n")
        for p in professores:
            if p.chmax != None:
                f.write(f'\t"{p.id()}" => {p.chmax},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("chmax1 :: Dict{String, Int64} = Dict{String, Int64}(\n")
        for p in professores:
            if p.chmax1 != None:
                f.write(f'\t"{p.id()}" => {p.chmax1},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")

        f.write("chmax2 :: Dict{String, Int64} = Dict{String, Int64}(\n")
        for p in professores:
            if p.chmax2 != None:
                f.write(f'\t"{p.id()}" => {p.chmax2},\n')
        f.seek(f.tell() - 2, 0)
        f.write("\n)\n\n")



import typer

from .construtor import criar_projeto

app = typer.Typer()


@app.command()
def init(nome_projeto: str):
    """Cria novo projeto"""
    criar_projeto(nome_projeto)
    print(f"✅ Projeto '{nome_projeto}' criado!")


# Outros comandos...
@app.command()
def run():
    """Executa processamento"""
    pass



#!/usr/bin/python
# coding=utf8

import logging
from datetime import date

from . import funcoes_escrita, funcoes_leitura


def check_course_sem(cs, S1INI, S2INI, verbosity=True):
    """ "Warn all the courses that start in semesters that are not
    their correct semesters."""

    logger = logging.getLogger("alforria")

    if not verbosity:
        logger = logging.getLogger()

    for c in cs:
        # Se for anual, nao tem problema
        if (c.dend - c.dini).days > 180:
            continue

        if (not (c.semestralidade == 1 and c.dini == S1INI)) and (
            not (c.semestralidade == 2 and c.dini == S2INI)
        ):
            logger.warn(
                "Semestralidade incompatível: {0:12s}\t{1:d}\t{2:10s}".format(
                    c.id(), c.semestralidade, date.isoformat(c.dini)
                )
            )


def check_g(cs, verbosity=True):
    """Warn all the courses in `cs` that do not have groups."""

    logger = logging.getLogger("alforria")

    if not verbosity:
        logger = logging.getLogger()

    logger.info("\nVerificando grupos.\n")

    for c in cs:
        if c.grupo is None:
            logger.info(
                "Turma %40s\t%6s\t%d\t%4d sem grupo.",
                c.nome,
                c.codigo,
                c.semestralidade,
                (17 * c.ch),
            )


def check_p_c(p, cs, params, verbosity=True):
    """This function checks if the addition of courses 'cs' creates an
    infeasible situation for professor 'p'. It does not check if the
    already assigned courses are feasible. Use 'check_p' for such
    test.

    Returns True if OK and False otherwise.

    """

    logger = logging.getLogger("alforria")

    if not verbosity:
        logger = logging.getLogger()

    ok = True

    if p.temporario:
        chmaxsem = int(params["chmax_temporario_semestral"])
        chmaxanual = int(params["chmax_temporario_anual"])
        chminanualgrad = int(params["chmin_graduacao"])
        chminanual = int(params["chmin_temporario_anual"])
    else:
        chmaxsem = int(params["chmax_efetivo_semestral"])
        chmaxanual = int(params["chmax_efetivo_anual"])
        chminanualgrad = int(params["chmin_graduacao"])
        chminanual = int(params["chmin_efetivo_anual"])

    if p.licenca1 or p.licenca2:
        chminanualgrad /= 2

    soma = p.chprevia1 + p.chprevia2
    soma1 = p.chprevia1
    soma2 = p.chprevia2

    chgrad = 0

    l_horarios = {
        (d, h, c.semestralidade) for (d, h) in c.horarios for c in p.turmas_a_lecionar
    }

    for c in cs:
        sem = c.semestralidade

        for d, h in c.horarios:
            if (d, h, sem) in l_horarios:
                logger.error(
                    "Professor %s - Disciplinas novas com horarios conflitantes: %s.",
                    p.nome(),
                    c.id(),
                )

                ok = False

            else:
                l_horarios.add((d, h, sem))

            if d < 7 and (
                (
                    (h == 15 or h == 16)
                    and (
                        (d + 1, 1, sem) in l_horarios
                        or (d + 1, 2, sem) in l_horarios
                        or (d + 1, 3, sem) in l_horarios
                    )
                )
                or (
                    (h == 13 or h == 14)
                    and ((d + 1, 1, sem) in l_horarios or (d + 1, 2, sem) in l_horarios)
                )
            ):
                logger.warn(
                    "Professor %s com menos de 11h entre dias consecutivos: turma %s.",
                    p.nome(),
                    c.id(),
                )

                ok = False

    for c in cs:
        ch = c.carga_horaria()
        soma += ch
        chgrad += ch

        if c.semestralidade == 1:
            soma1 += ch
        else:
            soma2 += ch

        if p.licenca1 and c.semestralidade == 1:
            logger.error("Professor %s - com licenca S1.", p.nome())

            ok = False

        if p.licenca2 and c.semestralidade == 2:
            logger.error("Professor %s - com licenca S2.", p.nome())

            ok = False

        if c.grupo is not None and c.grupo.id in p.inapto:
            logger.error("Professor %s - inapto para grupo %s.", p.nome(), c.grupo.id)

            ok = False

        for d, h in c.horarios:
            if p.impedimentos[h, d] == 1:
                logger.error(
                    "Professor %s - impedimento no dia e horario (%d, %d)",
                    p.nome(),
                    d,
                    h,
                )

                ok = False

    for t in p.turmas_a_lecionar:
        ch = t.carga_horaria()
        soma += ch
        chgrad += ch

        if t.semestralidade == 1:
            soma1 += ch
        else:
            soma2 += ch

        for d, h in t.horarios:
            if (d, h, t.semestralidade) in l_horarios:
                logger.error(
                    "Professor %s - nova disciplina conflita com ja atribuida %s",
                    p.nome(),
                    t.id(),
                )

                ok = False

    logger.debug(
        "%s carga total pre-atribuida com adicao de %s: %d.\n", p.nome(), c.id(), soma
    )

    if (
        (soma > chmaxanual and chgrad >= chminanualgrad)
        or (
            (chgrad >= chminanualgrad)
            and ((p.licenca1 and soma2 > chmaxsem) or (p.licenca2 and soma1 > chmaxsem))
        )
        or (not p.licenca1 and soma1 > chmaxsem)
        or (not p.licenca2 and soma2 > chmaxsem)
    ):
        logger.error(
            "Professor %s - adicao de %s excede carga horária máxima.", p.nome(), c.id()
        )

        ok = False

    return ok


def check_p(p, params):
    """This function checks if professor 'p' has a feasible set of
    attributed courses. Unlike the original 'check' function, it
    DOES NOT change any attribute of the professor.

    Returns True if OK or False otherwise.

    """

    logger = logging.getLogger("alforria")

    ok = True

    if p.temporario:
        chmaxsem = int(params["chmax_temporario_semestral"])
        chmaxanual = int(params["chmax_temporario_anual"])
        chminanualgrad = int(params["chmin_graduacao"])
        chminanual = int(params["chmin_temporario_anual"])
    else:
        chmaxsem = int(params["chmax_efetivo_semestral"])
        chmaxanual = int(params["chmax_efetivo_anual"])
        chminanualgrad = int(params["chmin_graduacao"])
        chminanual = int(params["chmin_efetivo_anual"])
        if p.pos:
            chminanualgrad = 8

    if p.licenca1 or p.licenca2:
        chminanualgrad /= 2

    soma = p.chprevia1 + p.chprevia2
    soma1 = p.chprevia1
    soma2 = p.chprevia2

    chgrad = 0

    l_horarios = []

    for t in p.turmas_a_lecionar:
        ch = t.carga_horaria()
        soma += ch
        chgrad += ch

        if t.semestralidade == 1:
            soma1 += ch
        else:
            soma2 += ch

        for d, h in t.horarios:
            if (d, h, t.semestralidade) in l_horarios:
                logger.error(
                    "Erro professor %s: Disciplina conflitante %s", p.nome(), t.id()
                )

                ok = False

            else:
                l_horarios.append((d, h, t.semestralidade))

            if p.impedimentos[h, d] == 1:
                logger.error(
                    "AVISO: Professor "
                    + str(p.nome())
                    + " no dia e horario "
                    + str((d, h))
                    + " com impedimento e disciplina "
                    + "pre-atribuida "
                    + str(t.id())
                    + "."
                )

                ok = False

        if p.licenca1 and t.semestralidade == 1:
            logger.error(
                "\tProfessor %s com disciplina %s pre-atribuida e " + "com licenca.",
                p.nome(),
                t.id(),
            )

            ok = False

        if p.licenca2 and t.semestralidade == 2:
            logger.error(
                "\tProfessor %s com disciplina %s pre-atribuida e " + "com licenca.",
                p.nome(),
                t.id(),
            )

            ok = False

        if t.grupo is not None and t.grupo.id in p.inapto:
            logger.info(
                "\tProfessor %s inapto para grupo %s de"
                + "disciplina pre-atribuida %s.",
                p.nome(),
                t.grupo.id,
                t.id(),
            )

            ok = False

    for t in p.turmas_a_lecionar:
        sem = t.semestralidade

        for d, h in t.horarios:
            if d < 7 and (
                (
                    (h == 15 or h == 16)
                    and (
                        (d + 1, 1, sem) in l_horarios
                        or (d + 1, 2, sem) in l_horarios
                        or (d + 1, 3, sem) in l_horarios
                    )
                )
                or (
                    (h == 13 or h == 14)
                    and ((d + 1, 1, sem) in l_horarios or (d + 1, 2, sem) in l_horarios)
                )
            ):
                logger.error(
                    "Professor %s com menos de 11h entre dias consecutivos: turma %s.",
                    p.nome(),
                    t.id(),
                )

                ok = False

    logger.debug("\t%s carga total pre-atribuida: %d.\n", p.nome(), soma)

    if chgrad < chminanualgrad:
        logger.error(
            "Professor %s - carga horaria insuficiente para graduacao." % p.nome()
        )

        ok = False

    if soma < chminanual:
        logger.error("Professor %s - carga horaria insuficiente." % p.nome())

        ok = False

    if (
        (soma > chmaxanual and chgrad >= chminanualgrad)
        or (
            (chgrad >= chminanualgrad)
            and ((p.licenca1 and soma2 > chmaxsem) or (p.licenca2 and soma1 > chmaxsem))
        )
        or (not p.licenca1 and soma1 > chmaxsem)
        or (not p.licenca2 and soma2 > chmaxsem)
    ):
        logger.error("Pre-atribuição excede carga horária máxima" + " de %s.", p.nome())

        ok = False

    return ok


############################################################################################################################
#########################################                LER AQUIVO DE FANTASMAS            ################################
############################################################################################################################


def ler_fantasmas(arquivo):
    """This function open the file having"""

    fantasmas = []
    with open(arquivo, "r") as fonte:
        for linha in fonte:
            tok = linha.split()

            fantasmas.append((int(tok[0]), int(tok[1])))

    return fantasmas


############################################################################################################################
#########################################        LIMPA E VERIFICA INCONSISTENCIAS           ################################
############################################################################################################################


def checkdata(professores, turmas, pre_atribuidas, S1INI, S2INI, FANTPATH):
    logger = logging.getLogger("alforria")

    constantes = funcoes_leitura.ler_conf("../config/constantes.cnf")

    fantasmas = ler_fantasmas(FANTPATH)

    # Verifica pre-atribuidas duplicadas
    # ----------------------------------

    logger.info("\n")
    logger.info("Checando pre-atribuidas duplicadas")
    logger.info("----------------------------------\n")

    check_duplicadas(pre_atribuidas)

    # Verifica impedimentos e etc
    # ---------------------------

    logger.info("\n")
    logger.info("Checando impedimentos e etc...")
    logger.info("------------------------------\n")

    p_ajustar = set()

    # Nao ha necessidade de remover impedimentos, pois o proprio
    # modelo ignora inconsistencias causadas por disciplinas
    # pre-atribuidas. Ou seja, uma pre-atribuida tem sempre
    # preferencia.
    for p, t in pre_atribuidas:
        for d, h in t.horarios:
            if p.impedimentos[h][d] == 1:
                logger.warning(
                    "AVISO: Professor "
                    + str(p.nome())
                    + " no dia e horario "
                    + str((d, h))
                    + " com impedimento e disciplina "
                    + "pre-atribuida "
                    + str(t.id())
                    + ". O impedimento NÃO será removido.\n"
                )

                # p.impedimentos[h][d]=0

                # Eh necessario reajustar os limites
                # quando removemos o impedimento
                # p_ajustar.add(p)

    for p in p_ajustar:
        p.ajustar()

    i = 0

    while i < len(pre_atribuidas) - 1:
        pi = pre_atribuidas[i][0]
        ti = pre_atribuidas[i][1]
        j = i + 1

        while j < len(pre_atribuidas):
            conflito = False
            pj = pre_atribuidas[j][0]
            tj = pre_atribuidas[j][1]

            if (
                pi == pj
                and ti != tj
                and int(ti.semestralidade) == int(tj.semestralidade)
            ):
                for x in ti.horarios:
                    if x in tj.horarios:
                        conflito = True

                        break
            if conflito:
                logger.warning(
                    "AVISO: Professor "
                    + pi.nome()
                    + " com 2 disciplinas pre-atribuidas conflitantes: "
                    + str(ti.id())
                    + " e "
                    + str(tj.id())
                    + "."
                )

                iguais = True

                for x in ti.horarios:
                    if x not in tj.horarios:
                        iguais = False
                        break
                for x in tj.horarios:
                    if x not in ti.horarios:
                        iguais = False
                        break
                if iguais:
                    logger.warning("\tAs turmas apresentam os mesmos horarios.")
                    k = 0
                    while k < len(turmas):
                        if (
                            int(turmas[k].codigo) == int(tj.codigo)
                            and int(turmas[k].turma) == int(tj.turma)
                            and turmas[k].semestralidade == tj.semestralidade
                        ):
                            tj.eh_cliente = True
                            ti.turmas_clientes.append(tj)
                            print(
                                "\t"
                                + str(turmas[k].id())
                                + " foi deletada e redefinida como fantasma."
                            )
                            del turmas[k]
                        else:
                            k += 1
                    del pre_atribuidas[j]
                else:
                    logger.error(
                        "\tERRO: Dados inconsistentes" + " para professor %s\n",
                        pi.nome(),
                    )

                    j += 1
            else:
                j += 1
        i += 1

    i = 0
    while i < len(pre_atribuidas) - 1:
        j = i + 1
        while j < len(pre_atribuidas):
            if pre_atribuidas[i][1] == pre_atribuidas[j][1]:
                del pre_atribuidas[j]
                print(
                    "Mesma disciplina pre atribuida em duplicata: "
                    + str(pre_atribuidas[i][1].id())
                    + " atribuida a "
                    + str(pre_atribuidas[i][0].id())
                    + " e "
                    + str(pre_atribuidas[i][1].id())
                    + ". Segunda associaçao removida."
                )
            else:
                j += 1
        i += 1

    # Verifica licencas
    # -----------------

    print("\n")
    print("Checando licencas")
    print("-----------------\n")

    check_licencas(pre_atribuidas)

    # Verifica inaptidoes
    # -------------------

    print("\n")
    print("Checando inaptidoes")
    print("-------------------\n")

    check_inaptidao(pre_atribuidas)

    # Verifica cargas horarias
    # ------------------------

    print("\n")
    print("Checando cargas horarias")
    print("------------------------\n")

    p_fantasma = check_ch(professores, turmas, pre_atribuidas, constantes)

    # Verifica disciplinas

    logger.info("\n")
    logger.info("Checando disciplinas")
    logger.info("--------------------\n")

    check_course_sem(turmas, S1INI, S2INI, verbosity=False)

    # Devolve a lista de professores fantasmas (para fins de relatorio)
    return p_fantasma


############################################################################################################################
#########################################                AUXILIARES                         ################################
############################################################################################################################


def check_duplicadas(pre_atribuidas):
    i = 0
    while i < len(pre_atribuidas) - 1:
        j = i + 1
        while j < len(pre_atribuidas):
            if pre_atribuidas[i][1] == pre_atribuidas[j][1]:
                print(
                    "AVISO: Disciplina "
                    + str(pre_atribuidas[i][1].id())
                    + " pre-atribuida em duplicata a:\n"
                    + "\t"
                    + str(pre_atribuidas[i][0].id())
                    + "\n"
                    + "\t"
                    + str(pre_atribuidas[j][0].id())
                    + ".\nSegunda associaçao removida."
                )
                del pre_atribuidas[j]
            else:
                j += 1
        i += 1


def check_ch(professores, turmas, pre_atribuidas, constantes):
    # Esta funcao supoe que as inconsistencias com licencas ja
    # foram removidas. Sua funcao eh consertar possiveis
    # incompatibilidades com a carga horaria.

    logger = logging.getLogger("alforria")

    p_fantasma = []
    for p in professores:
        if p.temporario:
            chmaxsem = int(constantes["chmax_temporario_semestral"])
            chmaxanual = int(constantes["chmax_temporario_anual"])
            chminanualgrad = int(constantes["chmin_graduacao"])
            chminanual = int(constantes["chmin_temporario_anual"])
        else:
            chmaxsem = int(constantes["chmax_efetivo_semestral"])
            chmaxanual = int(constantes["chmax_efetivo_anual"])
            chminanualgrad = int(constantes["chmin_graduacao"])
            chminanual = int(constantes["chmin_efetivo_anual"])

        # chminanualgrad = p.chmin

        if p.licenca1 or p.licenca2:
            chminanualgrad /= 2
            chminanual /= 2

        soma = p.chprevia1 + p.chprevia2
        soma1 = p.chprevia1
        soma2 = p.chprevia2

        chgrad = 0
        for p1, t1 in [(p2, t2) for (p2, t2) in pre_atribuidas if p == p2]:
            ch = t1.carga_horaria()
            soma += ch
            chgrad += ch

            if t1.semestralidade == 1:
                soma1 += ch
            else:
                soma2 += ch

        logger.debug("\t%s carga total pre-atribuida: %d.\n", p.nome(), soma)

        # Ha poucas disciplinas com 2 horas, entao eh melhor
        # remover o professor caso sua carga horaria chegue
        # proxima do maximo
        #
        # Professor eh removido quando
        #
        # - Deu o minimo de horas para a graduacao
        # - Estourou o maximo de horas OU eh da POS

        if (soma >= (chmaxanual - 2) and chgrad >= chminanualgrad) or (
            (chgrad >= chminanualgrad)
            and (
                (p.licenca1 and soma2 >= chmaxsem)
                or (p.licenca2 and soma1 >= chmaxsem)
                or (p.pos and soma >= chminanual)
            )
        ):
            logger.info(
                "Pre-atribuição atinge carga horária máxima"
                + " anual de %s. Professor e disciplinas "
                + "pre-atribuidas serao removidos do problema.",
                p.nome(),
            )

            p.carga_horaria = soma

            p_fantasma.append(p)

            for p1, t1 in [(p2, t2) for (p2, t2) in pre_atribuidas if p == p2]:
                p.add_course(t1)

                t1.add_professor(p)

                turmas.remove(t1)

                pre_atribuidas.remove((p1, t1))

                logger.info("\tRemovida turma %s", t1.id())

            continue

        # Se professor esta de licenca, zera carga horaria previa

        if p.licenca1:
            p.chprevia1 = 0

        if p.licenca2:
            p.chprevia2 = 0

        if chgrad < chminanualgrad:
            p.chmax = max(chmaxanual, soma + max(4, chminanualgrad - chgrad))

            if p.pos:
                p.chmax = max(chminanual, soma + max(4, chminanualgrad - chgrad))

            if not p.licenca1:
                p.chmax1 = max(chmaxsem, soma1 + max(4, chminanualgrad - chgrad))
                p.chmax = max(p.chmax, p.chmax1)
            if not p.licenca2:
                p.chmax2 = max(chmaxsem, soma2 + max(4, chminanualgrad - chgrad))
                p.chmax = max(p.chmax, p.chmax2)

        else:
            if not p.licenca1 and soma1 > chmaxsem:
                logger.info(
                    "Pre-atribuição viola carga horária "
                    + "máxima do 1o sem. de %s. Aumentando "
                    + "ch. maxima do semestre para %d",
                    p.nome(),
                    soma1,
                )
                p.chmax1 = soma1
                p.chmax = max(chmaxanual, soma1)

            if not p.licenca2 and soma2 > chmaxsem:
                logger.info(
                    "Pre-atribuição viola carga horária "
                    + "máxima do 2o sem. de %s. Aumentando "
                    + "ch. maxima do semestre para %d",
                    p.nome(),
                    soma2,
                )
                p.chmax2 = soma2
                p.chmax = max(chmaxanual, soma2)

    # Remove professores fantasmas
    for p in p_fantasma:
        p.fantasma = True
        professores.remove(p)

    return p_fantasma


def check_nao_atribuidas(turmas):
    logger = logging.getLogger("alforria")

    logger.info("\n")
    logger.info("Checando turmas sem professor")
    logger.info("-----------------------------\n")

    for t in turmas:
        if t.professor is None:
            logger.warning("\t%15s sem professor", t.id())


def check_licencas(pre_atribuidas):
    logger = logging.getLogger("alforria")

    for p, t in pre_atribuidas:
        if p.licenca1 and t.semestralidade == 1:
            logger.info(
                "\tProfessor %s com disciplina %s pre-atribuida e "
                + "com licenca.\n\t\tLicenca SEMESTRE 1 removida.",
                p.nome(),
                t.id(),
            )
            p.licenca1 = False

        if p.licenca2 and t.semestralidade == 2:
            logger.info(
                "\tProfessor %s com disciplina %s pre-atribuida e "
                + "com licenca.\n\t\tLicenca SEMESTRE 2 removida.",
                p.nome(),
                t.id(),
            )
            p.licenca2 = False


def check_inaptidao(pre_atribuidas):
    logger = logging.getLogger("alforria")

    for p, t in pre_atribuidas:
        if t.grupo is not None and t.grupo.id in p.inapto:
            logger.info(
                "\tProfessor %s inapto para grupo %s de "
                + "disciplina pre-atribuida %s.\n\t\t"
                + "Inaptidao removida.",
                p.nome(),
                t.grupo.id,
                t.id(),
            )

            p.inapto.remove(t.grupo.id)


############################################################################################################################
#########################################        IMPRIME ESTATISTICAS           ################################
############################################################################################################################


def estatisticas(professores, turmas):
    # Contabiliza carga horaria da graduacao

    chtotal = [0, 0, 0]  # chtotal[i]: charga horaria total do semestre i
    for t in turmas:
        chtotal[t.semestralidade] += t.carga_horaria()

    n_efetivos1 = 0
    n_efetivos2 = 0
    n_temporarios1 = 0
    n_temporarios2 = 0
    ch_previa_tt1 = 0
    ch_previa_tt2 = 0
    ch_efetivos = 0
    ch_temporarios = 0
    for p in professores:
        if p.temporario:
            if not p.licenca1:
                n_temporarios1 += 1
            if not p.licenca2:
                n_temporarios2 += 1
            ch_temporarios += p.carga_horaria_atrib()
        else:
            if not p.licenca1:
                n_efetivos1 += 1
            if not p.licenca2:
                n_efetivos2 += 1
            ch_efetivos += p.carga_horaria_atrib()
        ch_previa_tt1 += p.chprevia1
        ch_previa_tt2 += p.chprevia2
    print("\nEstatisticas" + "\n------------\n")
    print(
        "Coeficiente de dobro - 1o sem: {0:5.2f}".format(
            (chtotal[1] + ch_previa_tt1) / (n_efetivos1 + 2 * n_temporarios1)
        )
    )
    print(
        "Coeficiente de dobro - 2o sem: {0:5.2f}".format(
            (chtotal[2] + ch_previa_tt2) / (n_efetivos2 + 2 * n_temporarios2)
        )
    )
    print("Carga horaria:")
    print(
        "\t1o sem = "
        + str(chtotal[1])
        + ", professores: "
        + str(n_efetivos1)
        + " efetivos e "
        + str(n_temporarios1)
        + " temporarios"
    )
    print(
        "\t2o sem = "
        + str(chtotal[2])
        + ", professores: "
        + str(n_efetivos2)
        + " efetivos e "
        + str(n_temporarios2)
        + " temporarios"
    )
    print("Carga horaria previa total:")
    print("\t1o sem = " + str(ch_previa_tt1))
    print("\t2o sem = " + str(ch_previa_tt2))
    print("")
    print("Carga horaria total graduacao: " + str(chtotal[1] + chtotal[2]))
    print("\tEfetivos: " + str(ch_efetivos))
    print(
        "\tTemporarios: "
        + str(chtotal[1] + chtotal[2] - ch_efetivos)
        + " ("
        + str(ch_temporarios)
        + ")"
    )



"""Pacote Alforria"""

__version__ = "0.3.1"

import logging
import re
import sys

from prompt_toolkit import PromptSession
from prompt_toolkit.completion import WordCompleter, merge_completers

from . import check
from . import funcoes_escrita as escrita
from . import funcoes_leitura as leitura

_PATHS_PATH = "../config/paths.cnf"
_ALFCFG_PATH = "../config/alforria.cnf"
_CONST_PATH = "../config/constantes.cnf"

_alforria_completer = WordCompleter(
    [
        "attribute",
        "set_paths",
        "set_config",
        "load",
        "save",
        "moveto",
        "to_pdf",
        "check",
        "verbosity",
        "show",
        "remove",
        "report",
    ],
    ignore_case=True,
)

_session = None

# Configura nivel de saida

# Adiciona o vazio, caso necessario
logging.getLogger().addHandler(logging.NullHandler())

logger = logging.getLogger("alforria")

logger.addHandler(logging.StreamHandler())

logger.setLevel(logging.ERROR)

professores = None

grupos = None

turmas = None

pre_atribuidas = None

_professor_search_name = dict()

_course_search_id = dict()

_course_professor_search = dict()

_hd_course_search = dict()


def set_config_path(path: str):
    global _PATHS_PATH, _ALFCFG_PATH, _CONST_PATH
    _PATHS_PATH = path + "/paths.cnf"
    _ALFCFG_PATH = path + "/alforria.cnf"
    _CONST_PATH = path + "/constantes.cnf"


def __gen_stats__(params):
    """
    Utility function to calculate stats for courses and professors.
    """

    # Classes
    num_t = 0

    ch = [0, 0, 0]

    # Professors
    num_p_co = 0
    num_p_ef = 0
    num_p_ef_pos = 0

    ch_previa = [0, 0, 0]

    chminpe = [0, 0, 0]
    chmaxpe = [0, 0, 0]
    chminpc = [0, 0, 0]
    chmaxpc = [0, 0, 0]

    for t in turmas:
        # Conta as anuais como 2
        num_t += 1

        ch[2] += t.carga_horaria()

        if t.semestralidade == 1:
            ch[0] += t.carga_horaria()
        else:
            ch[1] += t.carga_horaria()

    for p in professores:
        ch_previa[0] += p.chprevia1
        ch_previa[1] += p.chprevia2
        ch_previa[2] += p.chprevia1 + p.chprevia2

        if p.temporario:
            num_p_co += 1
            chmaxsem = int(params["chmax_temporario_semestral"])
            chmaxanual = int(params["chmax_temporario_anual"])
            chminanual = int(params["chmin_temporario_anual"])
        else:
            num_p_ef += 1
            if p.pos:
                num_p_ef_pos += 1
            chmaxsem = int(params["chmax_efetivo_semestral"])
            chmaxanual = int(params["chmax_efetivo_anual"])
            chminanualgrad = int(params["chmin_graduacao"])
            chminanual = int(params["chmin_efetivo_anual"])

        if p.licenca1 and p.licenca2:
            continue

        if p.licenca1:
            if p.temporario:
                chminpc[1] += chminanual / 2
                chmaxpc[1] += chmaxsem
                chminpc[2] += chminanual / 2
                chmaxpc[2] += chmaxsem
            else:
                chminpe[1] += chminanual / 2
                chmaxpe[1] += chmaxsem
                chminpe[2] += chminanual / 2
                chmaxpe[2] += chmaxsem
        elif p.licenca2:
            if p.temporario:
                chminpc[0] += chminanual / 2
                chmaxpc[0] += chmaxsem
                chminpc[2] += chminanual / 2
                chmaxpc[2] += chmaxsem
            else:
                chminpe[0] += chminanual / 2
                chmaxpe[0] += chmaxsem
                chminpe[2] += chminanual / 2
                chmaxpe[2] += chmaxsem
        else:
            if p.temporario:
                chminpc[0] += chminanual / 2
                chmaxpc[0] += chmaxsem
                chminpc[1] += chminanual / 2
                chmaxpc[1] += chmaxsem
                chminpc[2] += chminanual
                chmaxpc[2] += chmaxanual
            else:
                chminpe[0] += chminanual / 2
                chmaxpe[0] += chmaxsem
                chminpe[1] += chminanual / 2
                chmaxpe[1] += chmaxsem
                chminpe[2] += chminanual
                chmaxpe[2] += chmaxanual

    return (
        [num_t],
        [num_p_ef],
        [num_p_ef_pos],
        [num_p_co],
        ch,
        ch_previa,
        chminpe,
        chmaxpe,
        chminpc,
        chmaxpc,
    )


def _report_(*args):
    """
    Create graphical and text reports, for data visualization.
    """

    import matplotlib.pyplot as plt
    import numpy as np

    global _CONST_PATH
    global professores, grupos, turmas

    params = leitura.ler_conf(_CONST_PATH)

    if len(args) == 0:
        logger.error("Uso: report [imp|t|g|ch]")

        return

    rtype = args[0]

    if rtype == "imp":
        h = np.zeros((17, 8))

        for p in professores:
            np.add(h, p.impedimentos, out=h)

        plt.matshow(h[1:17, 2:8], cmap=plt.cm.Reds)
        plt.colorbar()
        plt.xticks(np.arange(0, 6), ["S", "T", "Q", "Q", "S", "S"])
        plt.title("Impedimentos dos professores")
        plt.show()

    elif rtype == "t":
        hc = np.zeros((17, 8))

        for t in turmas:
            for d, h in t.horarios:
                hc[h, d] += 1.0

        plt.matshow(hc[1:18, 2:8], cmap=plt.cm.Reds)
        plt.colorbar()
        plt.title("Horarios das disciplinas ofertadas")
        plt.show()

    elif rtype == "g":
        hg = dict([(g.id, 0) for g in grupos])

        for p in professores:
            for gid in p.inapto:
                hg[gid] += 1

        plt.bar(hg.keys(), hg.values())
        plt.title("Impedimentos em grupos")
        plt.xticks(rotation=45)
        plt.show()

    elif rtype == "ch":
        nd, npe, npep, npc, ch, ch_previa, chminpe, chmaxpe, chminpc, chmaxpc = (
            __gen_stats__(params)
        )

        width = 0.25

        x = np.arange(0, 3)

        fig, axes = plt.subplots(ncols=2, nrows=1)

        axes[0].bar(x, ch, width, color="lightcoral", label="Demanda")
        axes[0].bar(
            x, ch_previa, width, color="indianred", label="Ch. Previa", bottom=ch
        )
        axes[0].bar(
            x + width, chminpe, width, color="cornflowerblue", label="Min. Efetivo"
        )
        axes[0].bar(
            x + width,
            chminpc,
            width,
            color="lightsteelblue",
            label="Min. Colaborador",
            bottom=chminpe,
        )
        axes[0].bar(
            x + 2 * width, chmaxpe, width, color="darkblue", label="Max. Efetivo"
        )
        axes[0].bar(
            x + 2 * width,
            chmaxpc,
            width,
            color="mediumblue",
            label="Max. Colaborador",
            bottom=chmaxpe,
        )
        axes[0].set_xticks(x + 1.25 * width)
        axes[0].set_xticklabels(["1S", "2S", "Total"])
        axes[0].set_title("Demanda e oferta")
        axes[0].legend()

        x = np.arange(0, 2)

        axes[1].bar(x[0], nd)
        axes[1].bar(x[1:], npe, label="Efetivos")
        axes[1].bar(x[1:], npep, label="Efetivos Pos", bottom=npe)
        axes[1].bar(x[1:], npc, label="Colaboradores", bottom=[npe[0] + npep[0]])
        axes[1].set_xticks(x)
        axes[1].set_xticklabels(["Turmas", "Professores"])
        axes[1].set_title("Turmas e professores")
        axes[1].legend()

        plt.show()

    else:
        logger.error("Uso: report [imp|t|g|ch]")


def _load_():
    """

    This function loads all the data.

    """

    global _PATHS_PATH, _ALFCFG_PATH
    global professores, grupos, turmas, pre_atribuidas
    global _professor_search_name, _course_search_id, _course_professor_search

    paths = leitura.ler_conf(_PATHS_PATH)
    configuracoes = leitura.ler_conf(_ALFCFG_PATH)

    # Carrega os grupos de disciplinas
    grupos = leitura.ler_grupos(paths["GRUPOSPATH"])

    # Carrega os professores e suas preferencias e ajusta os valores dados
    # às preferências para que fiquem entre 0 e 10.
    professores = leitura.ler_pref(
        paths["PREFPATH"], grupos, int(configuracoes["MAXIMPEDIMENTOS"])
    )

    # Updates the names of professors to the autocompleter. Also,
    # initializes the fast search for professors for a given time of
    # the week.

    for p in professores:
        p.ajustar()

    # Cria uma lista de busca dos professores por nome, para agilizar
    # a busca
    _professor_search_name = {p.nome(): p for p in professores}

    global _session

    if _session is not None:
        _session.completer = merge_completers(
            [_session.completer, WordCompleter(list(p.nome() for p in professores))]
        )

    # Carrega as turmas de disciplinas do ano e elimina as disciplinas
    # fantasmas (turmas com números diferentes que são, na verdade, a
    # mesma turma)

    turmas = leitura.ler_sar_csv(paths["SARPATH"], grupos)

    turmas = leitura.caca_fantasmas(paths["FANTPATH"], turmas)

    # Updates the names of the courses to the autocompleter

    if _session is not None:
        _session.completer = merge_completers(
            [_session.completer, WordCompleter(list(t.id() for t in turmas))]
        )

    for t in turmas:
        for d, h in t.horarios:
            if (d, h, t.semestralidade) in _hd_course_search:
                _hd_course_search[(d, h, t.semestralidade)].add(t)

            else:
                _hd_course_search[(d, h, t.semestralidade)] = {t}

    _course_professor_search = {
        t.id(): {p for p in professores if p.can_teach(t)} for t in turmas
    }

    # Cria uma lista de busca de turmas por id
    # (NOME_TURMA_SEMESTRALIDADE) para agilizar as buscas

    _course_search_id = {t.id(): t for t in turmas}

    # Carrega o arquivo de disciplinas pre-atribuidas
    pre_atribuidas = leitura.ler_pre_atribuidas(
        paths["ATRIBPATH"], paths["FANTPATH"], professores, turmas
    )


def _set_log_level_(level):
    """

    This function changes the log level for Alforria.

    """

    global logger

    v = int(cmds[1])

    if v < 1:
        logger.setLevel(logging.ERROR)

    elif v == 1:
        logger.setLevel(logging.INFO)

    else:
        logger.setLevel(logging.DEBUG)

    print("Changed logger level")


def _move_to_(*args):
    """Move a set o courses from a professor to another. Does not check
    overlapping courses.

    """

    global _professor_search_name, _course_search_id

    # Show correct usage of the function
    if len(args) < 3:
        logger.error(
            "Uso: moveto <professor origem> <professor destino> turma1 [turma2 turma3...]"
        )

        return

    # Check if both professors exist
    if args[0] not in _professor_search_name:
        logger.error("Nao encontrado docente: %s" % args[0])

        return

    if args[1] not in _professor_search_name:
        logger.error("Nao encontrado docente: %s" % args[1])

        return

    p_from = _professor_search_name[args[0]]

    p_to = _professor_search_name[args[1]]

    c = args[2:]

    # Only moves if the origin professor actually teaches all the
    # courses (that exist)

    tlist = _parse_turmas_(c, _course_search_id)

    for t in tlist:
        if t.professor is not p_from:
            logger.error(
                "Professor %s nao ministra turma %s. Nao ira mover disciplina alguma."
                % (p_from.nome(), t.id())
            )

            return

    for t in tlist:
        _remove_from_(t, p_from)

        _attribute_t_to_p(t, p_to)


def _parse_turmas_gen_(cstr, course_fast_index):
    """Cria um gerador, listando apenas as turmas existentes na lista
    'cstr'self.

    """

    for c in cstr:
        if c not in course_fast_index:
            logger.error("Turma %s nao encontrada. Ignorando." % c)

            continue

        yield course_fast_index[c]


def _parse_turmas_(cstr, course_fast_index):
    """Devolve uma lista de objetos do tipo Turma, criada a partir de uma
    lista de ids.

    """

    return list(_parse_turmas_gen_(cstr, course_fast_index))


def _attribute_t_to_p(t, p):
    """Utility function to add course to professor, checking previous
    attribuitions. Also, handles annual courses.

    """

    global _course_search_id

    # if t.vinculada and t.semestralidade == 2:

    #     logger.error("Nao permitido atribuir parte de disciplina anual %s -> %s.", (t.id(), p.nome()))

    #     return

    if t.professor is not None:
        if t.professor != p:
            logger.error("Turma %s ja atribuida a %s." % (t.id(), t.professor.nome()))

        return

    p.add_course(t)

    t.add_professor(p)

    # # Se a disciplina anual, sabemos S1 e S2 estao
    # # vinculados. Desta forma, procuramos S2 na lista de
    # # turmas

    # if t.vinculada and t.semestralidade == 1:

    #     cvinc = (t.id()).replace("S1", "S2")

    #     if cvinc not in _course_search_id:

    #         logger.error("Nao encontrada turma vinculada: %s", cvinc)

    #         return

    #     tvinc = _course_search_id[cvinc]

    #     p.add_course(tvinc)

    #     # Isso pode dar um problema se apenas o segundo semestre de
    #     # uma disciplina anual foi atribuido. Por enquanto isso eh
    #     # proibido.
    #     tvinc.add_professor(p)


def _remove_(*args):
    """
    Remove the given courses from the given professor.
    """

    global _professor_search_name, _course_search_id

    if len(args) < 2:
        logger.error("Uso: remove <professor> <turma1> [turma2 ...]")

        return

    name = args[0]

    cour = args[1:]

    if name not in _professor_search_name:
        logger.error("Nao encontrado docente: %s", name)

        return

    p = _professor_search_name[name]

    for tc in _parse_turmas_gen_(cour, _course_search_id):
        _remove_from_(tc, p)


def _remove_from_(t, p):
    """Remove turma 't' do professor 'p'. Se a turma eh anual, entao
    remove o segundo semestre tambem.

    TODO: Tratar o caso de disciplina anual quando o segundo semestre eh enviado.

    """

    t.remove_professor(p)

    p.remove_course(t)

    if t.vinculada and t.semestralidade == 1:
        cvinc = (t.id()).replace("S1", "S2")

        logger.warn("Atencao. Verifique se professor ministra %s.", cvinc)

        # if cvinc not in _course_search_id:

        #     logger.error("Nao encontrada turma vinculada: %s", cvinc)

        #     return

        # tvinc = _course_search_id[cvinc]

        # p.remove_course(tvinc)

        # tvinc.remove_professor(p)


def _attribute_(*args):
    """This function attributes courses to professors and professors to
    courses, according to the specified files or according to the arguments.

    """

    global \
        pre_atribuidas, \
        professores, \
        turmas, \
        _professor_search_name, \
        _course_search_id

    if pre_atribuidas is None or professores is None or turmas is None:
        print("Necessary to load data first.")

    nargs = len(args)

    if nargs == 0:
        for p, t in pre_atribuidas:
            _attribute_t_to_p(t, p)

            # # Nao adiciona o segundo semestre quando vinculada
            # if not t.vinculada or t.semestralidade == 1:

            #     _attribute_t_to_p(t, p)

    elif nargs >= 2:
        name = args[0]

        cour = args[1:]

        if name not in _professor_search_name:
            logger.error("Nao encontrado docente: %s", name)

            return

        p = _professor_search_name[name]

        for c in cour:
            if c not in _course_search_id:
                logger.error("Nao encontrada turma: %s", c)

                continue

            t = _course_search_id[c]

            _attribute_t_to_p(t, p)

    else:
        logger.error("Uso: attribute [professor turma1 turma2 ...].")


def _show_(*args):
    global professores, turmas, _professor_search_name, _course_search_id

    if len(args) != 1:
        print("Usage: show <professor ou turma>")

        return

    if professores is None or turmas is None:
        print("Necessary to load data first.")

        return

    name = args[0]

    if name not in _professor_search_name:
        if name not in _course_search_id:
            logger.error("Nao encontrado: %s.", name)

        else:
            print(_course_search_id[name])

    else:
        print(_professor_search_name[name].display())


def _save_csv_(*args):
    """
    This function saves the relations professors and classes.
    """

    global _ALFCFG_PATH
    global professores, turmas

    configuracoes = leitura.ler_conf(_ALFCFG_PATH)

    fname = configuracoes["RELDIR"] + "/atribuicoes.csv"

    if len(args) > 1:
        logger.error("Usage: save [fname]")

    elif len(args) == 1:
        fname = configuracoes["RELDIR"] + "/" + args[0]

    escrita.escreve_atribuicoes(professores, turmas, fname)


def _to_pdf_():
    global _ALFCFG_PATH
    global professores, turmas

    if professores is not None:
        configuracoes = leitura.ler_conf(_ALFCFG_PATH)

        prof_ord = sorted(professores, key=lambda x: x.nome())

        RELDIR = configuracoes["RELDIR"]

        escrita.cria_relatorio_geral(prof_ord, RELDIR)

        escrita.escreve_pre_atribuidas(
            professores, turmas, RELDIR + "pre_atribuidas.tsv"
        )

        escrita.escreve_atribuicoes(professores, turmas, RELDIR + "atribuicoes.csv")

        escrita.escreve_disciplinas(professores, turmas, RELDIR + "disciplinas.csv")

        escrita.write_ch_table(professores, RELDIR)

        print("Report created in directory %s" % RELDIR)

    else:
        print("Necessary to load data first.")


def _check_(*args):
    global _CONST_PATH, professores, _professor_search_name, _course_search_id

    constantes = leitura.ler_conf(_CONST_PATH)

    if professores is None:
        print("Necessary to load data first.")

        return

    if len(args) == 0:
        for p in professores:
            check.check_p(p, constantes)

    elif len(args) == 1:
        name = args[0]

        if name not in _professor_search_name:
            logger.error("Nao encontrado: %s.", name)

        else:
            check.check_p(_professor_search_name[name], constantes)

    elif len(args) >= 2:
        name = args[0]

        cour = args[1:]

        if name not in _professor_search_name:
            logger.error("Nao encontrado: %s.", name)

            return

        clist = [_course_search_id[c] for c in cour if c in _course_search_id]

        clist2 = [
            _course_search_id[(c.id()).replace("S1", "S2")]
            for c in clist
            if (c.vinculada and c.semestralidade == 1)
        ]

        clist.extend(clist2)

        if len(clist) == 0:
            logger.error("Nao encontrada nenhuma disciplina.")

            return

        check.check_p_c(_professor_search_name[name], clist, constantes)

    else:
        logger.error("Uso: check [professor [turma1 turma2 ...]]")


def _find_(*args):
    """Find available professors to teach a given class."""

    global _CONST_PATH
    global _course_professor_search, _course_search_id

    if len(args) != 1:
        logger.error("Uso: find turma")

        return

    name = args[0]

    if name not in _course_professor_search:
        logger.error("Turma nao encontrada: %s" % name)

        return

    constantes = leitura.ler_conf(_CONST_PATH)

    t = _course_search_id[name]

    s = _course_professor_search[name]

    # Deal with annual courses
    if t.vinculada and t.semestralidade == 1:
        name2 = (t.id()).replace("S1", "S2")

        s = s.intersection(_course_professor_search[name2])

    # Sort by group preference, filtering available professors
    for p in sorted(
        (pp for pp in s if check.check_p_c(pp, [t], constantes, verbosity=False)),
        reverse=True,
        key=lambda x: x.nome() if t.grupo is None else (10 - x.pref_grupos[t.grupo.id]),
    ):
        print(
            " %s: %d"
            % (p.nome(), -1 if t.grupo is None else (10 - p.pref_grupos[t.grupo.id]))
        )


def parse_command(command):
    """

    This function parses the commands and calls the correct functions.

    """

    global _PATHS_PATH
    global _ALFCFG_PATH
    global professores

    cmds = command.split()

    if len(cmds) > 0:
        if cmds[0] == "load":
            _load_()

        elif cmds[0] == "verbosity":
            if len(cmds) == 2:
                _set_log_level_(cmds[1])
            else:
                print("Usage: %s log_level" % cmds[0])

        elif cmds[0] == "set_paths":
            _PATHS_PATH = cmds[1]

        elif cmds[0] == "set_config":
            _ALFCFG_PATH = cmds[1]

        elif cmds[0] == "attribute":
            _attribute_(*cmds[1:])

        elif cmds[0] == "to_pdf":
            _to_pdf_()

        elif cmds[0] == "show":
            _show_(*cmds[1:])

        elif cmds[0] == "check":
            _check_(*cmds[1:])

        elif cmds[0] == "save":
            _save_csv_(*cmds[1:])

        elif cmds[0] == "remove":
            _remove_(*cmds[1:])

        elif cmds[0] == "report":
            _report_(*cmds[1:])

        elif cmds[0] == "find":
            _find_(*cmds[1:])

        elif cmds[0] == "moveto":
            _move_to_(*cmds[1:])

        else:
            print("Unknown command %s" % cmds[0])


def mainfunc():
    global _session

    _session = PromptSession(completer=_alforria_completer)

    while True:
        try:
            command = _session.prompt("> ")

        except KeyboardInterrupt:
            continue

        except EOFError:
            break

        else:
            try:
                parse_command(command)

            except Exception as e:
                logger.error("Unexpected error: " + str(e))

    print("Exiting.")


if __name__ == "__main__":
    mainfunc()



# coding=utf8

"""14-06-03 Joao:
Definicao da matricula como id do professor a fim de evitar duplicatas
Inclusao do item vinculada na classe Turma
Inclusao do item cliente/servidor na classe turma
Utilizacao do nome completo do professor, facilitara corresponder nomes em diferentes documentos, que podem apresenta-los inconsistentemente. Uma rotina nome foi adicionada.
"""

"""
14-08-08 Francisco
Conserto da funcao nome() de professor
Volta de id() de professor para nome, enquanto a matricula esta vazia
"""

"""
14-11-07 Francisco
Adicao de novos atributos e atualizacao da funcao que imprime o prof.
id() do professor agora eh a matricula
"""

"""
14-11-12 Francisco
funcao totex() na classe Professor
"""

"""
14-11-20 Francisco
- Melhoria na funcao totex()
"""

"""
14-11-26 Francisco
- Adicionado tratamento de licencas
- Preferencia por reuniao implica em impedimento para efetivo e insatisfacao maxima para temporario
"""

"""
14-11-11 Francisco
- Colocando '\' em caracteres nao numericos na hora de escrever o LaTeX
"""

"""
16-03-04 Francisco
- Ignorando notas de horarios com impedimento da hora do LaTeX
- Melhorando a saida do LaTeX
"""

"""
17-03-08 Francisco
- Adicionado atributo 'tel' e 'pos' em Professor
- Adicionado atributo carga horaria em Turma
- Modificando metodo carga_horaria em Turma
"""

import re

import numpy


class Paths:  ##################################################################################################
    def __init__(self):
        self.GRUPOSPATH = None
        self.PREFPATH = None
        self.SARPATH = None
        self.ATRIBPATH = None
        self.FANTPATH = None
        self.DATPATH = None
        self.SOLPATH = None
        self.preenchida = False


class Grupo:  ##################################################################################################
    def __init__(self):
        self.id = None
        self.canonico = False
        self.disciplinas = []

    def __str__(self):
        return str(self.id) + " " + str(self.canonico) + " " + str(self.disciplinas)


class Turma:  ##################################################################################################
    def __init__(self):
        self.codigo = None
        self.turma = None
        self.codigo = None
        self.nome = None
        self.semestralidade = None  # 1 ou 2
        self.horarios: list[
            tuple[numpy.int64, numpy.int64]
        ] = []  # Lista de pares (dia,horario)
        self.grupo: Grupo | None = None  # Objeto Grupo
        self.professor = None  # Objeto Professor ou Nome do Professor ????
        self.vinculada = False
        self.eh_cliente = False  # Indica se a turma eh uma turma cliente ou "fantasma" e tera suas aulas juntamente com outra
        self.turmas_clientes = []  # Lista de turmas que sao clientes desta
        self.pos = False
        self.ch = 0
        self.dini = None
        self.dend = None
        self.curso = ""

    # ----------------------------------------------------------------------------------------------------
    def id(self):
        return (
            str(self.codigo) + "_" + str(self.turma) + "_S" + str(self.semestralidade)
        )

    # ----------------------------------------------------------------------------------------------------
    def __str__(self):
        if self.professor is None:
            return (
                self.id()
                + " Sem professor "
                + " CH: "
                + str(self.ch)
                + " "
                + str(self.horarios)
            )
        else:
            return (
                self.id()
                + " "
                + self.professor.id()
                + " CH: "
                + str(self.ch)
                + " "
                + str(self.horarios)
            )

    # ----------------------------------------------------------------------------------------------------
    def carga_horaria(self):
        if self.ch > 0:
            return self.ch
        else:
            return len(self.horarios)

    # ----------------------------------------------------------------------------------------------------
    def add_professor(self, p):
        """Add a relation with professor 'p'."""
        self.professor = p

    def remove_professor(self, p):
        """Remove the relation with professor 'p'."""
        self.professor = None

    # ----------------------------------------------------------------------------------------------------
    def serialize(self):
        return {
            "codigo": self.codigo,
            "turma": self.turma,
            "codigo": self.codigo,
            "nome": self.nome,
            "semestralidade": self.semestralidade,
            "horarios": [
                {
                    "dia": dia.item(),
                    "horario": horario.item(),
                }
                for (dia, horario) in self.horarios
            ],
            "grupo": self.grupo.id if self.grupo is not None else "None",
            "professor": self.professor.matricula
            if self.professor is not None
            else "None",
            "vinculada": self.vinculada,
            "ehCliente": self.eh_cliente,
            "turmasClientes": [turma.codigo for turma in self.turmas_clientes],
            "pos": self.pos,
            "cargaHoraria": self.ch,
            "dataInicio": self.dini,
            "dataFim": self.dend,
            "curso": self.curso,
        }


class Professor:  ##################################################################################################
    def __init__(self):
        self.nome_completo = None
        self.matricula = None
        self.email = None
        self.tel = None
        self.chprevia1 = 0.0
        self.chprevia2 = 0.0
        self.licenca1 = False
        self.licenca2 = False
        self.discriminacao_chprevia = None
        self.temporario = False
        self.impedimentos = numpy.zeros((17, 8))
        self.peso_disciplinas = 0.0
        self.peso_disciplinas_bruto = 0.0
        self.peso_horario = 0.0
        self.peso_horario_bruto = 0.0
        self.peso_cargahor = 0.0
        self.peso_distintas = 0.0
        self.peso_janelas = 0.0
        self.peso_janelas_bruto = 0.0
        self.peso_numdisc = 0.0
        self.peso_manha_noite = 0.0
        self.inapto = []  # Lista de ids de grupos
        self.pref_grupos_bruto = {}  # Mapa id de grupo para preferencia
        self.pref_reuniao = False
        self.pref_janelas = False
        self.pref_horarios_bruto = numpy.zeros((17, 8))
        self.pref_grupos = {}
        self.pref_horarios = numpy.zeros((17, 8))
        self.lista_impedimentos = []
        self.chmax = None
        self.chmax1 = None
        self.chmax2 = None
        self.fantasma = False
        self.pos = False
        self.observacoes = ""

        # --------------------------------Valores lidos no arquivo de solucao---------------------
        self.turmas_a_lecionar = []
        self.carga_horaria = 0.0
        self.insatisfacao = 0.0
        self.insat_disciplinas = 0.0
        self.insat_cargahor = 0.0
        self.insat_numdisc = 0.0
        self.insat_horario = 0.0
        self.insat_distintas = 0.0
        self.insat_manha_noite = 0.0
        self.insat_janelas = 0.0

    def can_teach(self, t):
        """Return true if professor is able and available to teach class 't'."""

        return not (
            any(self.impedimentos[h, d] for (d, h) in t.horarios)
            or (t.grupo and t.grupo.id in self.inapto)
        )

    # ----------------------------------------------------------------------------------------------------
    def add_course(self, t):
        self.turmas_a_lecionar.append(t)

    def remove_course(self, t):
        if t in self.turmas_a_lecionar:
            self.turmas_a_lecionar.remove(t)

    # ----------------------------------------------------------------------------------------------------
    def carga_horaria_atrib(self):
        ch = 0

        for t in self.turmas_a_lecionar:
            ch += t.carga_horaria()

        return ch

    def _carga_horaria_s(self, semestre):
        ch = 0

        for t in [
            turma
            for turma in self.turmas_a_lecionar
            if turma.semestralidade is semestre
        ]:
            ch += t.carga_horaria()

        return ch

    def carga_horaria_s1(self):
        return self.chprevia1 + self._carga_horaria_s(1)

    def carga_horaria_s2(self):
        return self.chprevia2 + self._carga_horaria_s(2)

    # ----------------------------------------------------------------------------------------------------
    def carga_horaria_total(self):
        return self.chprevia1 + self.chprevia2 + self.carga_horaria_atrib()

    # ----------------------------------------------------------------------------------------------------
    def id(self):
        # O certo sera return self.matricula, mas ainda nao funciona
        return self.nome()

    # ----------------------------------------------------------------------------------------------------
    def nome(self):
        tok = self.nome_completo.split()
        n = ""
        for palavra in tok:
            if len(palavra) != 0:
                if n != "":
                    n += "_" + palavra
                else:
                    n += palavra
        return n

    # ----------------------------------------------------------------------------------------------------
    def __eq__(self, p):
        return True if (self.id() == p.id()) else False

    # ----------------------------------------------------------------------------------------------------
    def __hash__(self):
        return hash(self.id())

    # ----------------------------------------------------------------------------------------------------
    def __str__(self):
        s = (
            str(self.nome_completo)
            + " "
            + str(self.matricula)
            + " "
            + str(self.email)
            + "\n"
        )
        s += (
            "Temporario: "
            + str(self.temporario)
            + " Pref. Janelas: "
            + str(self.pref_janelas)
            + "\n"
        )
        s += (
            "Carga horaria previa - S1: "
            + str(self.chprevia1)
            + " S2: "
            + str(self.chprevia2)
            + "\n"
        )
        s += (
            "Pesos db="
            + str(self.peso_disciplinas_bruto)
            + " d="
            + str(self.peso_disciplinas)
            + " hb="
            + str(self.peso_horario_bruto)
            + " h="
            + str(self.peso_horario)
            + " c="
            + str(self.peso_cargahor)
            + " dist="
            + str(self.peso_distintas)
            + " jb="
            + str(self.peso_janelas_bruto)
            + " j="
            + str(self.peso_janelas)
            + " n="
            + str(self.peso_numdisc)
            + " mn="
            + str(self.peso_manha_noite)
            + "\n"
        )
        s += "Inapto=" + str(self.inapto) + "\n"
        s += "Impedimentos=\n" + str(self.impedimentos[1:17, 2:8]) + "\n"
        s += "Pref. Grupos Bruto=\n" + str(self.pref_grupos_bruto) + "\n"
        s += "Pref. Grupos=\n" + str(self.pref_grupos) + "\n"
        s += "Pref. Horarios Bruto\n" + str(self.pref_horarios_bruto[1:17, 2:8]) + "\n"
        s += "Pref. Horarios\n" + str(self.pref_horarios[1:17, 2:8])
        return s

    # ----------------------------------------------------------------------------------------------------
    def ajustar(self):
        # Ajuste do impedimento caso seja um titular que queira participar da reuniao
        if self.pref_reuniao and not self.temporario:
            for h in range(1, 6):
                self.impedimentos[h, 3] = 1

        # Ajuste da preferencia por horarios
        maximo = 0
        for j in range(2, 8):
            fim = 17
            if j == 7:
                fim = 11
            for i in range(1, fim):
                if (
                    self.pref_horarios_bruto[i, j] > maximo
                    and not self.impedimentos[i, j]
                ):
                    maximo = self.pref_horarios_bruto[i, j]
        minimo = 10
        for j in range(2, 8):
            fim = 17
            if j == 7:
                fim = 11
            for i in range(1, fim):
                if (
                    self.pref_horarios_bruto[i, j] < minimo
                    and not self.impedimentos[i, j]
                ):
                    minimo = self.pref_horarios_bruto[i, j]
        if maximo == minimo:
            self.peso_horario = 0.0
        else:
            self.peso_horario = self.peso_horario_bruto
            for j in range(2, 8):
                for i in range(1, 17):
                    self.pref_horarios[i, j] = (
                        -10
                        * (self.pref_horarios_bruto[i, j] - maximo)
                        / (maximo - minimo)
                    )
            # Se o temporario deseja participar da reuniao, entao coloca insatisfacao maxima no horario
            if self.pref_reuniao and self.temporario:
                for h in range(1, 6):
                    self.pref_horarios[h, 3] = 10
        # Ajuste da preferencia por grupos
        maximo = 0
        for g in self.pref_grupos_bruto.keys():
            if self.pref_grupos_bruto[g] > maximo and g not in self.inapto:
                maximo = self.pref_grupos_bruto[g]
        minimo = 10
        for g in self.pref_grupos_bruto.keys():
            if self.pref_grupos_bruto[g] < minimo and g not in self.inapto:
                minimo = self.pref_grupos_bruto[g]
        if maximo == minimo:
            self.peso_disciplinas = 0.0
        else:
            self.peso_disciplinas = self.peso_disciplinas_bruto
            for g in self.pref_grupos_bruto.keys():
                self.pref_grupos[g] = (
                    -10 * (self.pref_grupos_bruto[g] - maximo) / (maximo - minimo)
                )

    # ----------------------------------------------------------------------------------------------------
    def display(self):
        s = self.nome()
        if self.pos:
            s += " P"
        s += " (Ch. previa: %d (1S) %d (2S), " % (
            int(self.chprevia1),
            int(self.chprevia2),
        )
        s += "Ch. total: %d (1S) %d (2S))\n" % (
            self.carga_horaria_s1(),
            self.carga_horaria_s2(),
        )
        for t in self.turmas_a_lecionar:
            s += " "
            s += str(t)
            s += "\n"
        return s

    # ----------------------------------------------------------------------------------------------------
    def totex(self, arquivo=None):
        """
        Criam um arquivo .tex cujo nome eh a matricula, contendo os comandos tex para analisar os resultados.
        O arquivo precisa ser inserido na estrutura de um documento latex, via include, por exemplo.
        """

        if arquivo is None:
            arquivo = str(self.matricula).zfill(6) + ".tex"

        with open(arquivo, "w") as f:
            f.write(
                "\\section*{"
                + str(self.nome_completo)
                + "\\hfill "
                + str(self.matricula).zfill(6)
                + "}\n"
            )
            f.write(
                "\\begin{itemize}\n \\item "
                + re.sub("([_])", "\\\\\\1", self.email)
                + "\n"
            )
            if self.temporario:
                f.write("\\item Temporário \n")
            else:
                f.write("\\item Efetivo \n")

            f.write("\\item Aplicar carga horária mínima: ")
            if self.pos:
                f.write("Sim\n")
            else:
                f.write("Não\n")

            f.write(
                "\\item Carga horária prévia: " + str(self.chprevia1 + self.chprevia2)
            )
            if len(self.discriminacao_chprevia.strip()) > 0:
                f.write(" (" + self.discriminacao_chprevia + ")")
            f.write("\n")
            f.write(
                "\\item Carga horária anual (prévia + atribuída): "
                + str(int(self.carga_horaria_total()))
                + "\n"
            )

            if self.fantasma:
                f.write("\\item Satisfação: --\n")
            else:
                f.write(
                    "\\item Satisfação: {0:5.2f}\n".format(10.0 - self.insatisfacao)
                )
            f.write("\\begin{center} \\begin{tabular}{|l||r|r|r|r|r|r|r|} \\hline\n")
            f.write("& Disc. & Num. disc. & Disc. distintas & Hor. & Carga hor. & ")
            if self.pref_janelas:
                f.write("Janelas")
            else:
                f.write("Hor. compactos")
            f.write(" & Manhã e noite \\\\ \midrule\n")
            f.write(
                "Pesos & {0:5.2f} & {1:5.2f} & {2:5.2f} & {3:5.2f} & {4:5.2f} & {5:5.2f} & {6:5.2f} \\\\\n".format(
                    self.peso_disciplinas,
                    self.peso_numdisc,
                    self.peso_distintas,
                    self.peso_horario,
                    self.peso_cargahor,
                    self.peso_janelas,
                    self.peso_manha_noite,
                )
            )
            if self.fantasma:
                f.write("Satisfação & -- & -- & -- & -- & -- & -- & -- \\\\\n")
            else:
                f.write(
                    "Satisfação & {0:5.2f} & {1:5.2f} & {2:5.2f} & {3:5.2f} & {4:5.2f} & {5:5.2f} & {6:5.2f} \\\\\n".format(
                        10.0 - self.insat_disciplinas,
                        10.0 - self.insat_numdisc,
                        10.0 - self.insat_distintas,
                        10.0 - self.insat_horario,
                        10.0 - self.insat_cargahor,
                        10.0 - self.insat_janelas,
                        10.0 - self.insat_manha_noite,
                    )
                )
            f.write("\\hline \\end{tabular} \\end{center}\n")
            f.write("\\end{itemize}")

            ini = (176, 176, 176)
            dir = (-11, 76, -61)

            f.write("\\begin{multicols}{2}\n \\scriptsize")
            for s in range(1, 3):
                f.write("\\begin{center} \\begin{tabular}{|c|c|c|c|c|c|c|}\\toprule\n")
                f.write(
                    "\\multicolumn{7}{|c|}{"
                    + str(s)
                    + "$^\\circ$ semestre ("
                    + str(self._carga_horaria_s(s))
                    + "h/sem)} \\\\ \\midrule\n"
                )
                f.write("& S & T & Q & Q & S & S \\\\ \\midrule\n")
                for i in range(1, 17):
                    f.write(str(i))
                    for j in range(2, 8):
                        if (
                            self.impedimentos[i, j]
                            or (self.licenca1 and s == 1)
                            or (self.licenca2 and s == 2)
                            or (j == 7 and i >= 11)
                        ):
                            f.write("& \\cellcolor[gray]{1} ")
                        else:
                            m = (10.0 - self.pref_horarios[i, j]) / 10.0
                            f.write("& \\cellcolor[RGB]{")
                            for k in range(0, 3):
                                f.write(str(int(ini[k] + m * dir[k])))
                                if k < 2:
                                    f.write(",")
                            f.write("}")
                        for t in self.turmas_a_lecionar:
                            if t.semestralidade == s and (j, i) in t.horarios:
                                f.write(str(t.codigo) + " " + str(t.turma))

                    f.write("\\\\ \\midrule \n")

                f.write("\\end{tabular} \\end{center}\n\n")

            f.write("\\end{multicols}\n")
            f.write("\\begin{multicols}{2}\n")
            f.write("\\begin{center} \\begin{tabular}{|lm{6cm}|}\n")
            f.write(
                "\\multicolumn{2}{c}{Disciplinas a lecionar} \\\\ \\midrule \\midrule\n"
            )
            f.write(
                "\\multicolumn{2}{|c|}{1$^\\circ$ Semestre} \\\\ \\midrule \\midrule\n"
            )
            for t in [i for i in self.turmas_a_lecionar if i.semestralidade == 1]:
                f.write(str(t.codigo) + " & " + t.nome + "\\\\ \\midrule\n")
            f.write("\\midrule\n")
            f.write(
                "\\multicolumn{2}{|c|}{2$^\\circ$ Semestre} \\\\ \\midrule \\midrule\n"
            )
            for t in [i for i in self.turmas_a_lecionar if i.semestralidade == 2]:
                f.write(str(t.codigo) + " & " + t.nome + "\\\\ \\midrule\n")
            f.write("\\end{tabular} \\end{center} \\vfill\\columnbreak\n")
            f.write("\\begin{center} \\begin{tabular}{|lr|}\n")
            f.write(
                "\\multicolumn{2}{c}{Preferência de grupos} \\\\ \\midrule \\midrule\n"
            )

            for g, p in sorted(self.pref_grupos.items(), key=lambda x: x[1]):
                if g not in self.inapto:
                    f.write(g + " & " + str(round(10 - p, 1)) + "\\\\ \\midrule \n")
            for g in self.inapto:
                f.write(g + "& INAPTO \\\\ \\midrule \n")

            f.write("\\end{tabular} \\end{center}\n \\end{multicols}\n")
            f.write("{\\normalsize \\textbf{OBS}: " + self.observacoes + "}")

    # ----------------------------------------------------------------------------------------------------
    def att_totex(self, arquivo=None):
        """
        Criam um arquivo .tex cujo nome eh a matricula, contendo apenas as atribuicoes.
        O arquivo precisa ser inserido na estrutura de um documento latex, via include, por exemplo.
        """

        if arquivo is None:
            arquivo = str(self.matricula).zfill(6) + ".tex"

        with open(arquivo, "w") as f:
            f.write(
                "\\section*{"
                + str(self.nome_completo)
                + "\\hfill "
                + str(self.matricula).zfill(6)
                + "}\n"
            )

            f.write("\\begin{multicols}{2}\n \\scriptsize")
            for s in range(1, 3):
                f.write("\\begin{center} \\begin{tabular}{|c|c|c|c|c|c|c|}\\toprule\n")
                f.write(
                    "\\multicolumn{7}{|c|}{"
                    + str(s)
                    + "$^\\circ$ semestre} \\\\ \\midrule\n"
                )
                f.write("& S & T & Q & Q & S & S \\\\ \\midrule\n")
                for i in range(1, 17):
                    f.write(str(i))
                    for j in range(2, 8):
                        f.write("& ")

                        for t in self.turmas_a_lecionar:
                            if t.semestralidade == s and (j, i) in t.horarios:
                                f.write(str(t.codigo) + " " + str(t.turma))

                    f.write("\\\\ \\midrule \n")

                f.write("\\end{tabular} \\end{center}\n\n")

            f.write("\\end{multicols}\n")
            f.write("\\begin{multicols}{2}\n")
            f.write("\\begin{center} \\begin{tabular}{|lm{6cm}|}\n")
            f.write(
                "\\multicolumn{2}{c}{Disciplinas a lecionar} \\\\ \\midrule \\midrule\n"
            )
            f.write(
                "\\multicolumn{2}{|c|}{1$^\\circ$ Semestre} \\\\ \\midrule \\midrule\n"
            )
            for t in [i for i in self.turmas_a_lecionar if i.semestralidade == 1]:
                f.write(str(t.codigo) + " & " + t.nome + "\\\\ \\midrule\n")
            f.write("\\midrule\n")
            f.write(
                "\\multicolumn{2}{|c|}{2$^\\circ$ Semestre} \\\\ \\midrule \\midrule\n"
            )
            for t in [i for i in self.turmas_a_lecionar if i.semestralidade == 2]:
                f.write(str(t.codigo) + " & " + t.nome + "\\\\ \\midrule\n")
            f.write("\\end{tabular} \\end{center} \\vfill\\columnbreak\n")
            f.write("\\end{multicols}\n")

    # ----------------------------------------------------------------------------------------------------
    def serialize(self):
        return {
            "nomeCompleto": self.nome_completo,
            "matricula": self.matricula,
            "email": self.email,
            "telefone": self.tel,
            "chPrevia1": self.chprevia1,
            "chPrevia2": self.chprevia2,
            "licenca1": self.licenca1,
            "licenca2": self.licenca2,
            "discriminacaoChPrevia": self.discriminacao_chprevia,
            "temporario": self.temporario,
            "impedimentos": [
                [x for x in y] for y in self.impedimentos
            ],  # acho que pode ser um array de tuplas ?
            "pesoDisciplinas": self.peso_disciplinas,
            "pesoDisciplinasBruto": self.peso_disciplinas_bruto,
            "pesoHorario": self.peso_horario,
            "pesoHorarioBruto": self.peso_horario_bruto,
            "pesoCargaHoraria": self.peso_cargahor,
            "pesoDistintas": self.peso_distintas,
            "pesoJanelas": self.peso_janelas,
            "pesoJanelasBruto": self.peso_janelas_bruto,
            "pesoNumDisc": self.peso_numdisc,
            "pesoManhaNoite": self.peso_manha_noite,
            "inapto": self.inapto,
            "prefGruposBruto": self.pref_grupos_bruto,
            "prefReuniao": self.pref_reuniao,
            "prefJanelas": self.pref_janelas,
            "prefHorariosBruto": [
                [dia for dia in horario] for horario in self.pref_horarios_bruto
            ],
            "prefGrupos": self.pref_grupos,
            "prefHorarios": [
                [dia for dia in horario] for horario in self.pref_horarios
            ],
            "listaImpedimentos": self.lista_impedimentos,
            "chMax": self.chmax,
            "chMax1": self.chmax1,
            "chMax2": self.chmax2,
            "fantasma": self.fantasma,
            "pos": self.pos,
            "observacoes": self.observacoes,
            # # --------------------------------Valores lidos no arquivo de solucao---------------------
            "turmasALecionar": [t.codigo for t in self.turmas_a_lecionar],
            "cargaHoraria": self.carga_horaria,
            "insatisfacao": self.insatisfacao,
            "insatDisciplinas": self.insat_disciplinas,
            "insatCargaHoraria": self.insat_cargahor,
            "insatNumDisc": self.insat_numdisc,
            "insatHorario": self.insat_horario,
            "insatDistintas": self.insat_distintas,
            "insatManhaNoite": self.insat_manha_noite,
            "insatJanelas": self.insat_janelas,
        }


##################################################################################################



# coding=utf8

import re
from datetime import date

import numpy
import pandas as pd

from . import classes, funcoes_gerais


def sar292_to_df(sarfile):
    rx = re.compile(
        r"^[\s]*[\d]{1,3}[\s]*2024[\s]*DMA[\s]*([\d]{3,5})[\s]*([\d]{1,3})[^\d]+([\d]{1,3})"
    )

    with open(sarfile, "r") as fp:
        for l in fp:
            m = rx.match(l)

            if m != None:
                entries.append([])

    return pd.DataFrame(
        entries, columns=["curso", "codigo", "turma", "matriculados", "professor"]
    )


def ler_curso_do_sar097(sarfile):
    """
    Le um arquivo txt convertido de um SAR097 e devolve um dicionario codigo -> curso.
    """

    rx = re.compile(r"^[\s]*([\d]{3,5}) ([\d]{3})")
    rxc = re.compile(r" [\d]{1,2} ([^\d]+)")

    d = {}

    with open(sarfile, "r") as fp:
        for l in fp:
            m = rx.match(l)

            if m != None:
                c = m[1]
                t = m[2]

                curso = rxc.search(l)[1].strip()

                if c not in d:
                    d[c] = curso

                # print("{cod:6s} {turma:3s} {curso:20s}".format(cod=c, turma=t, curso=curso))

    return d


####################################################################################################################
####################                  Funcao              caca_fantasmas                   #########################
####################################################################################################################
# Remove os fantasmas da lista turmas
def caca_fantasmas(arquivo, turmas):
    with open(arquivo, "r") as fonte:
        print("\nRemovendo disciplinas fantasmas")
        print("-------------------------------\n")

        for num_linha, linha in enumerate(fonte, 1):
            fantasma = linha.split("\t")
            fantasma_inexistente = True
            i = 0

            while i < len(turmas):
                if int(turmas[i].codigo) == int(fantasma[0]) and int(
                    turmas[i].turma
                ) == int(fantasma[1]):
                    print("Turma " + turmas[i].id() + " removida")
                    del turmas[i]
                    fantasma_inexistente = False
                else:
                    i += 1
            if fantasma_inexistente:
                print(
                    "\nAVISO: Linha "
                    + str(num_linha)
                    + " - Este fantasma nao existe:"
                    + fantasma[0]
                    + "_"
                    + fantasma[1]
                    + ".\n"
                )
    return turmas


def ler_solucao_gurobi_jump(professores, turmas, arquivo):
    with open(arquivo, "r") as fonte:
        for linha in fonte:
            # Pula as linhas que nao setam variaveis
            if not "[" in linha:
                continue

            a = re.split("[\[\]]", linha)

            b = a[1].split(",")
            professor = b[0]
            variavel = a[0]

            if variavel == "carga_horaria":
                for p in professores:
                    if p.nome() == professor:
                        p.carga_horaria = float(linha.split()[-1])
            elif variavel == "x":
                turmaid = b[1].strip("'")
                if float(a[-1].strip()) > 0.9:
                    for t in turmas:
                        if t.id() == turmaid:
                            break
                    for p in professores:
                        if p.nome() == professor:
                            p.turmas_a_lecionar.append(t)
                            break
            elif variavel == "insat":
                for p in professores:
                    if p.nome() == professor:
                        p.insatisfacao = float(linha.split()[-1])
            elif variavel == "insat_disciplinas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_disciplinas = float(linha.split()[-1])
            elif variavel == "insat_cargahor":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_cargahor = float(linha.split()[-1])
            elif variavel == "insat_numdisc":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_numdisc = float(linha.split()[-1])
            elif variavel == "insat_horario":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_horario = float(linha.split()[-1])
            elif variavel == "insat_distintas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_distintas = float(linha.split()[-1])
            elif variavel == "insat_manha_noite":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_manha_noite = float(linha.split()[-1])
            elif variavel == "insat_janelas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_janelas = float(linha.split()[-1])


####################################################################################################################
####################    Funcao                 ler_solucao              e auxiliares      #########################
####################################################################################################################
def ler_solucao(professores, turmas, arquivo):
    with open(arquivo, "r") as fonte:
        for linha in fonte:
            # Pula as linhas que nao setam variaveis
            if not "(" in linha:
                continue

            a = re.split("[\[\]]", linha)

            b = a[1].split(",")
            professor = b[0]
            variavel = a[0]

            if variavel == "carga_horaria":
                for p in professores:
                    if p.nome() == professor:
                        p.carga_horaria = float(linha.split()[-1])
            elif variavel == "x":
                turmaid = b[1].strip("'")
                if float(a[-1].strip()) > 0.9:
                    for t in turmas:
                        if t.id() == turmaid:
                            break
                    for p in professores:
                        if p.nome() == professor:
                            p.add_course(t)
                            t.add_professor(p)
                            break
            elif variavel == "insat":
                for p in professores:
                    if p.nome() == professor:
                        p.insatisfacao = float(linha.split()[-1])
            elif variavel == "insat_disciplinas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_disciplinas = float(linha.split()[-1])
            elif variavel == "insat_cargahor":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_cargahor = float(linha.split()[-1])
            elif variavel == "insat_numdisc":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_numdisc = float(linha.split()[-1])
            elif variavel == "insat_horario":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_horario = float(linha.split()[-1])
            elif variavel == "insat_distintas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_distintas = float(linha.split()[-1])
            elif variavel == "insat_manha_noite":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_manha_noite = float(linha.split()[-1])
            elif variavel == "insat_janelas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_janelas = float(linha.split()[-1])


####################################################################################################################
####################    Funcao                 ler_solucao_cbc          e auxiliares      #########################
####################################################################################################################
def ler_solucao_cbc(professores, turmas, arquivo):
    with open(arquivo, "r") as fonte:
        for linha in fonte:
            # Pula as linhas que nao setam variaveis
            if not "(" in linha:
                continue

            a = re.split("[()]", linha.split()[1])
            b = a[1].split(",")

            professor = b[0]
            variavel = a[0]

            if variavel == "carga_horaria":
                for p in professores:
                    if p.nome() == professor:
                        p.carga_horaria = float(linha.split()[-2])
            elif variavel == "x":
                turmaid = b[1].strip("'")
                if linha.split()[-2] == "1":
                    for t in turmas:
                        if t.id() == turmaid:
                            break
                    for p in professores:
                        if p.nome() == professor:
                            p.turmas_a_lecionar.append(t)
                            break
            elif variavel == "insat":
                for p in professores:
                    if p.nome() == professor:
                        p.insatisfacao = float(linha.split()[-2])
            elif variavel == "insat_disciplinas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_disciplinas = float(linha.split()[-2])
            elif variavel == "insat_cargahor":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_cargahor = float(linha.split()[-2])
            elif variavel == "insat_numdisc":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_numdisc = float(linha.split()[-2])
            elif variavel == "insat_horario":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_horario = float(linha.split()[-2])
            elif variavel == "insat_distintas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_distintas = float(linha.split()[-2])
            elif variavel == "insat_manha_noite":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_manha_noite = float(linha.split()[-2])
            elif variavel == "insat_janelas":
                for p in professores:
                    if p.nome() == professor:
                        p.insat_janelas = float(linha.split()[-2])


####################################################################################################################
####################    Funcao                 ler_grupos                e auxiliares      #########################
####################################################################################################################
def ler_grupos(arquivo):  # Le os grupos do arquivo de grupos
    grupos = []
    with open(arquivo, "r") as fonte:
        for linha in fonte:
            if not linha.isspace():
                tok = linha.split()
                g = classes.Grupo()
                g.id = funcoes_gerais.uniformize(tok[1])
                if tok[0] == "C":
                    g.canonico = True
                for i in tok[2:]:
                    g.disciplinas.append(i)
                grupos.append(g)
    return grupos


####################################################################################################################
####################    Funcao                 ler_sar                   e auxiliares      #########################
####################################################################################################################
def get_sar_version(sarfile):
    """
    Devolve um inteiro associado com a versao do SAR `sarfile` enviado.
    """

    with open(sarfile, "r") as fp:
        for linha in fp:
            if "DISC" in linha:
                if "TIPO" in linha:
                    if "VAGAS" in linha:
                        return 3

                    return 2

                return 1


def sar_vale(linha):  # Define se a linha lida do SAR possui informacoes relevantes
    if len(linha) < 10:
        return False
    tok = linha.split()
    if len(tok) < 10:
        return False
    if tok[0] == "DISC":
        return False
    if tok[-9] in ["MARINGA", "CIANORTE"]:
        return True
    # Caso com horario indisponivel
    if tok[-7] in ["MARINGA", "CIANORTE"]:
        return True


# ----------------------------------------------------------------------------------------------------------------------
def sar_primaria(
    linha,
):  # Uma linha primaria e aquela que comeca uma nova turma, essa funcao as identifica
    if not sar_vale(linha):
        return False
    tok = linha.split()

    # Elimina o caso que a linha comeca com o dia e o caso em que
    # comeca com T,P,T-P
    if len(tok[0]) > 2 and tok[0].isdigit():
        return True

    return False


# ----------------------------------------------------------------------------------------------------------------------
def horario_valido(d, h):
    return d >= 1 and d <= 7 and h >= 1 and h <= 16


# ----------------------------------------------------------------------------------------------------------------------
def copia_turmas(origem):
    destino = classes.Turma()
    destino.codigo = origem.codigo
    destino.turma = origem.turma
    destino.nome = origem.nome
    destino.semestralidade = origem.semestralidade
    destino.dini = origem.dini
    destino.dend = origem.dend
    destino.horarios = list(origem.horarios)  # Cria uma copia por valor
    destino.grupo = origem.grupo
    destino.professor = origem.professor
    destino.vinculada = origem.vinculada
    destino.ch = origem.ch
    destino.curso = origem.curso
    return destino


def _csv_to_dataframe(sarfile):
    df = pd.read_csv(
        sarfile,
        header=None,
        parse_dates=[15, 16],
        delimiter="\s*,\s*",
        engine="python",
        comment="#",
    )

    # Transforma em pares
    df[18] = df[[6, 7]].apply(lambda x: [(x[6], x[7])], axis=1)

    # Agrega tudo

    agg_map = {i: "first" for i in range(2, 18) if i not in [6, 7, 14]}
    agg_map[18] = "sum"

    return df.groupby([0, 1, 14]).agg(agg_map)


def _get_grupo(codigo, semestralidade, grupos):
    codigo_s = codigo + "_S{}".format(semestralidade)

    for g in grupos:
        if codigo in g.disciplinas or codigo_s in g.disciplinas:
            return g

    return None


def _load_turmas(df_sar, grupos):
    turmas = []

    for r in df_sar.itertuples():
        t = classes.Turma()

        t.codigo = str(r.Index[0])
        t.turma = str(r.Index[1])
        t.semestralidade = 1 if r.Index[2] == "S1" else 2

        t.grupo = _get_grupo(t.codigo, t.semestralidade, grupos)

        t.dini = r._11.date()
        t.dend = r._12.date()

        t.curso = "" if pd.isna(r._13) else r._13

        if any([(not horario_valido(d, h)) for (d, h) in r._14]):
            print(
                "AVISO: Ignorando turma %s: horario invalido (%d, %d)."
                % (t.codigo, d, h)
            )

            continue

        t.horarios.extend(r._14)

        t.nome = r._1

        t.ch = int(r._9)

        turmas.append(t)

    # Procura vinculadas

    for t in [t for t in turmas if (t.dend - t.dini).days > 180]:
        if t.vinculada:
            continue

        for tt in [tt for tt in turmas if not tt.vinculada]:
            if (
                t.codigo == tt.codigo
                and t.turma == tt.turma
                and t.semestralidade != tt.semestralidade
            ):
                t.vinculada = True
                tt.vinculada = True

    return turmas


def ler_sar_csv(arquivo, grupos):
    return _load_turmas(_csv_to_dataframe(arquivo), grupos)


# ----------------------------------------------------------------------------------------------------------------------
def ler_sar(
    arquivo, grupos, cursos={}
):  # arquivo: arquivo do SAR, grupos: lista dos objetos grupos
    turmas = []
    anual = False
    invalida = False
    t = None

    sar_version = get_sar_version(arquivo)

    with open(arquivo, "r") as fonte:
        for linha in fonte:
            tok = linha.split()
            if sar_primaria(linha):
                # Se a ultima linha era valida, guarde-a
                if t is not None and not invalida:
                    turmas.append(t)

                # se a ultima disciplina lida foi anual, sua correspondente no segundo semestre sera criada
                if anual and not invalida:
                    t = copia_turmas(turmas[-1])
                    t.semestralidade = 2
                    temp = str(t.codigo) + "_S2"
                    for g in grupos:
                        if temp in g.disciplinas:
                            t.grupo = g
                            break
                    turmas.append(t)

                invalida = False
                anual = False
                semestres = []
                t = classes.Turma()
                t.codigo = tok[0]
                t.turma = tok[1]

                # Handle the semester
                l = list(map(int, tok[-3].split("/")))
                t.dini = date(l[2], l[1], l[0])

                l = list(map(int, tok[-1].split("/")))
                t.dend = date(l[2], l[1], l[0])

                if tok[-4] == "A":
                    t.semestralidade = 1
                    anual = True
                    t.vinculada = True
                else:
                    anual = False
                    if tok[-4] == "S1" or tok[-4] == "M1":
                        t.semestralidade = 1
                    else:
                        t.semestralidade = 2

                # Cuida do caso em que nao veio a
                # carga horaria da disciplina
                ntok = -10
                if "." in tok[-6] or "," in tok[-6]:
                    ntok = -12

                d, h = (int(tok[ntok]), int(tok[ntok + 1]))

                if not horario_valido(d, h):
                    print(
                        "AVISO: Ignorando turma %s: horario invalido (%d, %d)."
                        % (t.codigo, d, h)
                    )
                    invalida = True
                    continue

                t.horarios.append((d, h))

                # Ajusta nome de acordo com versao do SAR
                if sar_version == 2:
                    ntok -= 1
                elif sar_version == 3:
                    ntok -= 3

                t.nome = ""
                for i in tok[2:ntok]:
                    t.nome = t.nome + " " + i

                temp = str(t.codigo)
                if anual:
                    temp += "_S" + str(t.semestralidade)
                for g in grupos:
                    if temp in g.disciplinas:
                        t.grupo = g
                        break

                # Para a carga horaria, precisamos
                # remover a virgula
                # O novo SAR vem com
                # '.' ao invés de ',', entao esse
                # codigo é para compatibilidade
                t.ch = 0
                if "," in tok[-6]:
                    t.ch = int(tok[-6].split(",")[0])

                if "." in tok[-6]:
                    t.ch = int(tok[-6].split(".")[0])

                # Determina o curso
                if t.codigo in cursos:
                    t.curso = cursos[t.codigo]

                else:
                    print("AVISO: Disciplina {} sem curso.".format(t.id()))

            else:
                if sar_vale(linha) and not invalida:
                    # Novo SAR, agora tem um T ou T-P ou P em cada linha ou,
                    # pior, com vagas e matriculados.
                    if sar_version == 1:
                        d, h = (int(tok[0]), int(tok[1]))
                    elif sar_version == 2:
                        d, h = (int(tok[1]), int(tok[2]))
                    elif sar_version == 3:
                        d, h = (int(tok[3]), int(tok[4]))

                    if not horario_valido(d, h):
                        print(
                            "AVISO: Ignorando turma %s: horario invalido (%d, %d)."
                            % (t.codigo, d, h)
                        )
                        invalida = True
                        continue

                    t.horarios.append((d, h))

        # fim do loop grande
        if t is not None and not invalida:
            turmas.append(t)

        # se a ultima disciplina (de todas) lida foi anual, sua correspondente no segundo semestre sera criada
        if anual and not invalida:
            t = copia_turmas(turmas[len(turmas)])
            t.semestralidade = 2
            temp = str(t.codigo) + "_S2"
            for g in grupos:
                if temp in g.disciplinas:
                    t.grupo = g
                    break
            turmas.append(t)

        return turmas


############################################################################################################################
#########################################                LER PREFERENCIAS                   ################################
############################################################################################################################
converter_horario = {
    "07:45 - 09:25": (1, 2),
    "09:40 - 12:10": (3, 4, 5),
    "13:30 - 15:10": (6, 7),
    "15:20 - 17:50": (8, 9, 10),
    "17:50 - 19:30": (11, 12),
    "19:30 - 21:10": (13, 14),
    "21:20 - 23:00": (15, 16),
}
# ----------------------------------------------------------------------------------------------------------------------
converter_dia = {"SEG": 2, "TER": 3, "QUA": 4, "QUI": 5, "SEX": 6, "SAB": 7}
# ----------------------------------------------------------------------------------------------------------------------
converter_codigo_de_horario = {
    1: (1, 2),
    2: (3, 4, 5),
    3: (6, 7),
    4: (8, 9, 10),
    5: (11, 12),
    6: (13, 14),
    7: (15, 16),
}
# ----------------------------------------------------------------------------------------------------------------------
converter_periodo = {
    "MANHA": (1, 2, 3, 4, 5),
    "TARDE": (6, 7, 8, 9, 10, 11, 12),
    "NOITE": (13, 14, 15, 16),
}
# ----------------------------------------------------------------------------------------------------------------------
converter_preferencia = {
    "NAO GOSTARIA": 0,
    "DESGOSTARIA LEVEMENTE": 2.5,
    "INDIFERENTE": 5,
    "GOSTARIA LEVEMENTE": 7.5,
    "GOSTARIA": 10,
}


# ----------------------------------------------------------------------------------------------------------------------
def ler_pref(form, grupos, max_impedimentos):
    professores = []
    with open(form, "r", encoding="utf-8") as f:
        for l in f:
            p = classes.Professor()
            tokens = iter(l.split("\t"))
            next(tokens)  # Pula timestamp
            next(tokens)  # Pula email duplicado
            p.nome_completo = funcoes_gerais.uniformize(next(tokens))  # Identificacao
            p.matricula = int(next(tokens))  # Identificacao Unica
            # Jeito tosco de remover duplicatas
            # TODO: melhorar isso
            for i in professores:
                if i.matricula == p.matricula:
                    print(
                        "AVISO: Professores "
                        + i.nome()
                        + " e "
                        + p.nome()
                        + " com mesma matrícula. Eliminando entrada antiga.\n"
                    )
                    professores.remove(i)
            p.email = next(tokens)
            p.tel = next(tokens)
            chp1 = next(tokens)
            chp2 = next(tokens)
            if chp1.strip() == "":
                chp1 = "0"
            if chp2.strip() == "":
                chp2 = "0"
            p.chprevia1 = int(chp1)
            p.chprevia2 = int(chp2)
            p.discriminacao_chprevia = next(tokens)
            w = funcoes_gerais.uniformize(next(tokens))  # Licença
            if "PRIMEIRO" in w:
                p.licenca1 = True
            if "SEGUNDO" in w:
                p.licenca2 = True
            if "ANUAL" in w:
                continue
            w = funcoes_gerais.uniformize(next(tokens))  # Temporario
            if w == "TEMPORARIO":
                p.temporario = True
            # Le qual o programa de pos
            p.programa_pos = funcoes_gerais.uniformize(next(tokens))
            # Professor tem reducao de carga para a pos?
            p.pos = (
                "PMA" in p.programa_pos
                or "PCM" in p.programa_pos
                or "PEQ" in p.programa_pos
                or "PROFMAT" in p.programa_pos
                or "OUTRO" in p.programa_pos
            )
            # Pesos das disciplinas
            p.peso_disciplinas_bruto = float(next(tokens))
            p.peso_horario_bruto = float(next(tokens))
            p.peso_cargahor = float(next(tokens))
            p.peso_distintas = float(next(tokens))
            p.peso_numdisc = float(next(tokens))
            p.peso_manha_noite = float(next(tokens))
            p.peso_janelas_bruto = float(next(tokens))
            # Inaptidao em grupos
            w = funcoes_gerais.uniformize(next(tokens))
            if len(w) > 0:
                p.inapto = w.split(", ")
            # Preferencia por grupos
            for g in grupos:
                if g.canonico:
                    s = funcoes_gerais.uniformize(next(tokens))
                    if len(s) > 0:  # se a preferencia foi preenchida
                        p.pref_grupos_bruto[g.id] = converter_preferencia[s]
                    else:  # se não, atribuir default 5
                        p.pref_grupos_bruto[g.id] = converter_preferencia["INDIFERENTE"]
            # Reuniao departamento
            w = funcoes_gerais.uniformize(next(tokens))
            if "SIM" in w:
                p.pref_reuniao = True
            # Preferencia compacto/esparso (define o respectivo peso como zero se nao especificado)
            w = funcoes_gerais.uniformize(next(tokens))
            p.peso_janelas = p.peso_janelas_bruto
            if "ESPARSOS" in w:
                p.pref_janelas = True
            elif "COMPACTOS" not in w:
                p.peso_janelas = (
                    0  # Zera peso_janelas se a Preferencia nao for informada
                )
            for i in range(0, max_impedimentos):
                dia = funcoes_gerais.uniformize(next(tokens))
                periodo = funcoes_gerais.uniformize(next(tokens))
                justificativa = funcoes_gerais.uniformize(next(tokens))
                # guarda os dados do impedimento como string para conferencia posterior
                p.lista_impedimentos.append(dia + ", " + periodo + ", " + justificativa)
                # se o dia e periodo foram corretamente preenchidos, marca a respectiva posicao na matriz como 1
                if (
                    len(dia) > 0
                    and not dia.isspace()
                    and dia != "SEM IMPEDIMENTO"
                    and len(periodo) > 0
                    and not periodo.isspace()
                    and periodo != "SEM IMPEDIMENTO"
                ):
                    for h in converter_periodo[periodo]:
                        p.impedimentos[h, converter_dia[dia]] = 1

            # Perguntas do curso - ignorar ate chegar na grade horaria
            w = funcoes_gerais.uniformize(next(tokens))
            while not ("DETALHADO" in w or "RESUMIDO" in w):
                w = funcoes_gerais.uniformize(next(tokens))

            # Comeca o prechimento das tabelas de horario
            m = numpy.zeros((15, 8))
            if "DETALHADO" in w:
                for d in range(2, 8):
                    periodos = ["MANHA", "TARDE", "NOITE"]
                    if d == 7:
                        periodos = ["MANHA", "TARDE"]
                    for i in periodos:
                        pref = funcoes_gerais.uniformize(next(tokens))
                        for h in converter_periodo[i]:
                            if len(pref) == 0:
                                pref = "INDIFERENTE"
                            p.pref_horarios_bruto[h, d] = converter_preferencia[pref]
                for i in range(0, 5):  # Pula as posicoes em branco do formulario
                    try:
                        next(tokens)
                    except StopIteration:
                        pass
            else:
                for i in range(
                    0, 3 * 5 + 2
                ):  # pula as posicoes em branco do formulario
                    next(tokens)
                for i in [
                    "MANHA",
                    "TARDE",
                    "NOITE",
                ]:  # le as preferencias para os dias da semana
                    pref = funcoes_gerais.uniformize(next(tokens))
                    for h in converter_periodo[i]:
                        for d in range(2, 7):  # grava uma copia em cada dia da semana
                            if len(pref) == 0:
                                pref = "INDIFERENTE"
                            p.pref_horarios_bruto[h, d] = converter_preferencia[pref]
                for i in ["MANHA", "TARDE"]:  # lendo as preferencias do sabado
                    pref = funcoes_gerais.uniformize(next(tokens))
                    for h in converter_periodo[i]:
                        if len(pref) == 0:
                            pref = "INDIFERENTE"
                        p.pref_horarios_bruto[h, d + 1] = converter_preferencia[pref]
            # Tudo o que vier depois daqui eh considerado comentario.
            for obs in tokens:
                p.observacoes += obs

            professores.append(p)
    return professores


############################################################################################################################
#########################################                LER PRE-ATRIBUIDAS                 ################################
############################################################################################################################
def ler_pre_atribuidas(arquivo, arquivo_de_fantasmas, professores, turmas):
    """
    Devolve uma lista de pares (professor,turma) pre atribuidos
    """

    print("\nCarregando pre-atribuidas")
    print("---------------------------\n")

    fantasma = []

    with open(arquivo_de_fantasmas, "r") as fant:
        for linha in fant:
            tok = linha.split("\t")

            fantasma.append((tok[0], tok[1]))

    delim = "\t"

    with open(arquivo, "r") as f:
        for l in f:
            if len(l) > 0 and not l.isspace() and r"," in l:
                delim = r","
                break

    with open(arquivo, "r") as f:
        linha = 1

        pre_atribuidas = []

        for l in f:
            if len(l) < 0 or l.isspace():
                continue

            tokens = iter(l.split(delim))

            try:
                matricula = int(next(tokens))

                nome_professor = next(tokens)

                cod_disciplina = next(tokens)

                turma = next(tokens)

                nome_disciplina = next(tokens).rstrip()

                next(tokens)

                semestralidade = int(next(tokens).rstrip()[1])

            except Exception as e:
                s = (
                    "AVISO: Ignorando linha "
                    + str(linha)
                    + " em PRE-ATRIBUIDAS. Erro: "
                    + str(e)
                )

                print(s)

                linha += 1

                continue

            for p in professores:
                if p.matricula == matricula:
                    encontrada = False
                    for t in turmas:
                        # Nao precisa se preocupar com a semestralidade, pois
                        # o codigo e a turma vao bater duas vezes, em caso de
                        # disciplina anual
                        if (
                            t.codigo == cod_disciplina
                            and t.turma == turma
                            and t.semestralidade == semestralidade
                        ):
                            encontrada = True
                            if not (cod_disciplina, turma) in fantasma:
                                pre_atribuidas.append((p, t))
                    if not encontrada:
                        if not (cod_disciplina, turma) in fantasma:
                            s = (
                                "AVISO: Disciplina "
                                + cod_disciplina
                                + " turma "
                                + turma
                                + " ("
                                + nome_disciplina
                                + ") nao encontrada."
                            )
                            print(s)
                    break
            else:
                s = "AVISO: Linha " + str(linha) + ": matricula " + str(matricula) + " "
                s += "(professor " + nome_professor + ")" + " nao encontrada!"
                print(s)

            linha += 1

    return pre_atribuidas


############################################################################################################################
#########################################                LER AQUIVO DE CONFIGURACAO         ################################
############################################################################################################################


def ler_conf(arquivo):
    """
    Le o arquivo de configuracao e devolve um mapa {PARAMETRO:VALOR}com os parametros lidos
    """

    param = {}
    linha = 1

    with open(arquivo, "r") as f:
        for l in f:
            if l.isspace() or l.startswith("#"):
                continue

            tokens = l.split()

            if len(tokens) == 2:
                param[tokens[0]] = tokens[1]
            else:
                print("\nLER_CONF.PY Erro na linha " + str(linha) + "\n")

            linha += 1

    return param
